/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import be.cardon.xml.XSDDateTime;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.util.Date;
import java.util.GregorianCalendar;
import java.net.URI;


/**DOM-based implementation of {@code CRLRefType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMCRLRef extends DOMStructure{
    
    public final static String TAG_DIGESTALGANDVALUE = "DigestAlgAndValue";
    public final static String TAG_CRLIDENTIFIER = "CRLIdentifier";
    public final static String TAG_ISSUER = "Issuer";
    public final static String TAG_ISSUETIME = "IssueTime";
    public final static String TAG_NUMBER = "Number";
    public final static String TAG_URI = "URI";
    
    /**DigestAldAndValue, never null.*/
    DOMDigestAlgAndValue digestAlgAndValue;
    
    /**CRLIdentifier, may be null.*/
    CRLIdentifier cRLIdentifier;
    
    /** Creates a new instance of DOMCRLRef.
     *@param digestAlgAndValue, required (not null)
     *@param cRLIdentifier, optional (may be null)
     *@throws NullPointerException if digestAlgAndValue is null.
     */
    public DOMCRLRef(
            DOMDigestAlgAndValue digestAlgAndValue,
            CRLIdentifier cRLIdentifier){
        if(digestAlgAndValue==null){
            throw new NullPointerException("digestAlgAndValue is null.");
        }
        this.digestAlgAndValue = digestAlgAndValue;
        this.cRLIdentifier = cRLIdentifier;
    }
    
    /**Unmarshal a DOMCRLRef.*/
    public DOMCRLRef(Element elem, DOMCryptoContext context) throws MarshalException{
        try{
            String xadesNS = XadESContext.getXadesNS(context);
            DOMUtils.checkName(elem, null, xadesNS);
            
            Element digestAldAndValueElem = DOMUtils.getNextSiblingElement(elem);
            DOMUtils.checkName(digestAldAndValueElem, TAG_DIGESTALGANDVALUE, xadesNS);
            this.digestAlgAndValue = new DOMDigestAlgAndValue(digestAldAndValueElem, context);
            
            Element cRLIdentifierElem = DOMUtils.getNextSiblingElement(digestAldAndValueElem);
            if(cRLIdentifierElem!=null){
                DOMUtils.checkName(cRLIdentifierElem, TAG_CRLIDENTIFIER, xadesNS);
                
                Element issuerElem = DOMUtils.getFirstChildElement(cRLIdentifierElem);
                DOMUtils.checkName(issuerElem, TAG_ISSUER, xadesNS);
                String issuer = issuerElem.getTextContent();
                
                Element issueTimeElem = DOMUtils.getNextSiblingElement(issuerElem);
                DOMUtils.checkName(issueTimeElem, TAG_ISSUETIME, xadesNS);
                GregorianCalendar issueTime = XSDDateTime.parseDateTime(
                        issueTimeElem.getTextContent()).toGregorianCalendar();
                        
                Element numberElem = DOMUtils.getNextSiblingElement(issueTimeElem);
                Integer number =null;
                if(numberElem!=null){
                    DOMUtils.checkName(numberElem, TAG_NUMBER, xadesNS);
                    number = new Integer(numberElem.getTextContent());
                }
                
                URI uri = XSDAnyURI.parseURI(
                        cRLIdentifierElem.getAttributeNS(null, TAG_URI));
                
                this.cRLIdentifier = new CRLIdentifier(issuer, issueTime, number, uri);
            }
            
            
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
        
    }
    
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element cRLRefElem = DOMUtils.createElement(doc, localName, 
                xadesNS, xadesPrefix);
        
        digestAlgAndValue.marshal(cRLRefElem, TAG_DIGESTALGANDVALUE, context);
        if(cRLIdentifier!=null){
            Element cRLIdentifierElem = DOMUtils.createElement(doc, TAG_CRLIDENTIFIER, 
                xadesNS, xadesPrefix);
            
            Element issuerElem = DOMUtils.createElement(doc, TAG_ISSUER, 
                xadesNS, xadesPrefix);
            issuerElem.appendChild(doc.createTextNode(
                    cRLIdentifier.getIssuer()));
            cRLIdentifierElem.appendChild(issuerElem);
            
            Element issueTimeElem = DOMUtils.createElement(doc, TAG_ISSUETIME, 
                xadesNS, xadesPrefix);
            issueTimeElem.appendChild(doc.createTextNode(
                    XSDDateTime.printDateTime(cRLIdentifier.getIssueTime())));
            cRLIdentifierElem.appendChild(issueTimeElem);
            
            if(cRLIdentifier.getNumber()!=null){
                Element numberElem = DOMUtils.createElement(doc, TAG_ISSUER, 
                xadesNS, xadesPrefix);
                numberElem.appendChild(doc.createTextNode(
                        cRLIdentifier.getNumber().toString()));
                cRLIdentifierElem.appendChild(numberElem);
            }
            if(cRLIdentifier.getUri()!=null){
                cRLIdentifierElem.setAttributeNS(null, TAG_URI, 
                        XSDAnyURI.printURI(cRLIdentifier.getUri()));
            }
            cRLRefElem.appendChild(cRLIdentifierElem);
        }
        parent.appendChild(cRLRefElem);
    }
    
    /**CRLIdentifier.*/
    public class CRLIdentifier{
        
        /**Issuer, never null, never empty.*/
        String issuer;
        
        /**IssueTime, never null.*/
        GregorianCalendar issueTime;
        
        /**Number, may be null.*/
        Integer number;
        
        /**URI, may be null.*/
        URI uri;
        
        /**Constructor.
         *@param issuer, required (not empty or null).
         *@param issueTime, never null.
         *@param number, optional (may be null).
         *@param uri, optional (may be null).
         *@throws NullPointerException if issuer or issueTime is null.
         *@throws IllegalArgumentException if issuer is empty.
         *
         */
        public CRLIdentifier(
                String issuer,
                GregorianCalendar issueTime,
                Integer number,
                URI uri){
            if(issuer==null){
                throw new NullPointerException("Issuer may not be null.");
            }
            if(issuer.length()<1){
                throw new IllegalArgumentException("Issuer string may not be empty.");
            }
            if(issueTime==null){
                throw new NullPointerException("IssueTime may not be null.");
            }
            
            this.issuer = issuer;
            this.issueTime = issueTime;
            this.number = number;
            this.uri = uri;
        }
        
        /**Returns the issue time. Never null.*/
        public GregorianCalendar getIssueTime() {
            return issueTime;
        }
        
        /**Returns the issuer name, never null, never empty.*/
        public String getIssuer() {
            return issuer;
        }
        
        /**Returns the number, may be null.*/
        public Integer getNumber() {
            return number;
        }
        
        /**Returns the URI, may be null.*/
        public URI getUri() {
            return uri;
        }
        
        
    }
    
}
