/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMX509IssuerSerial;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.net.URI;
import be.cardon.xml.XSDAnyURI;


/**DOM-based implementation of {@code CertIDListType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMCertIDList extends DOMStructure{
    
    public final static String TAG_CERT = "Cert";
    public final static String TAG_CERTDIGEST = "CertDigest";
    public final static String TAG_ISSUERSERIAL = "IssuerSerial";
    public final static String TAG_URI = "URI";
    
    /**Cert list, may be neither empty, neither null.*/
    List<CertID> certs = new ArrayList<CertID>();
    
    /** Creates a new instance of DOMCertIDList */
    public DOMCertIDList(List<CertID> certs) {
        if(certs!=null){
            if(certs.size()==0){
                throw new IllegalArgumentException("certs list is empty.");
            }
            this.certs = certs;
        }
    }
    
    /**Unmarshal a CertIDList.*/
    public DOMCertIDList(Element elem, DOMCryptoContext context)
    throws MarshalException{
        try{
            String xadesNS = XadESContext.getXadesNS(context);
            DOMUtils.checkName(elem, null, xadesNS);
            
            Element child = DOMUtils.getFirstChildElement(elem);
            while(child!=null){
                DOMUtils.checkName(child, TAG_CERT, xadesNS);
                
                Element certDigestElem = DOMUtils.getFirstChildElement(child);
                DOMUtils.checkName(certDigestElem, TAG_CERTDIGEST, xadesNS);
                DOMDigestAlgAndValue certDigest =
                        new DOMDigestAlgAndValue(certDigestElem, context);
                
                Element issuerSerialElem = DOMUtils.getNextSiblingElement(certDigestElem);
                DOMUtils.checkName(issuerSerialElem, TAG_ISSUERSERIAL, xadesNS);
                DOMX509IssuerSerial issuerSerial =
                        new DOMX509IssuerSerial(issuerSerialElem);
                
                String URIString = elem.getAttributeNS(null, TAG_URI);
                URI uri= XSDAnyURI.parseURI(URIString); //returns null if empty string
                certs.add(new CertID(certDigest, issuerSerial, uri));
                child = DOMUtils.getNextSiblingElement(child);
            }
            
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }

    public List<DOMCertIDList.CertID> getCerts() {
        return certs;
    }

    public void marshal(Node parent, String localName, DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element certIDListElem = DOMUtils.createElement(doc, localName,
                xadesNS, xadesPrefix);
        Iterator<CertID> it = certs.iterator();
        while(it.hasNext()){
            CertID certID = it.next();
            Element certIDElem = DOMUtils.createElement(doc, TAG_CERT,
                    xadesNS, xadesPrefix);
            certID.getCertDigest().marshal(certIDElem, TAG_CERTDIGEST, context);
            certID.getIssuerSerial().marshal(certIDElem,
                    dsPrefix,
                    xadesPrefix,
                    TAG_ISSUERSERIAL, 
                    xadesNS,
                    context);
            certIDListElem.appendChild(certIDElem);
        }
        parent.appendChild(certIDListElem);
    }
    
    public String toString(){
            try{   
            String dump = "";
            Iterator it = this.getCerts().iterator();
            while(it.hasNext()){
                dump += "[Cert:\n"+it.next().toString()+"]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
    
    
    public static class CertID{
        
        /** CertDigest, never null.*/
        DOMDigestAlgAndValue certDigest;
        
        /** issuerSerial, never null.*/
        DOMX509IssuerSerial issuerSerial;
        
        /** optional URI, may be null.*/
        URI uri;
        
        /**Creates a new instance
         *@param certDigest the digest algorithm and value
         *@param issuerSerial the issuer serial of the certificate
         *@papam uri source, may be null.
         */
        public CertID(DOMDigestAlgAndValue certDigest,
                DOMX509IssuerSerial issuerSerial,
                URI uri){
            if(certDigest==null){
                throw new NullPointerException("certDigest cannot be null.");
            }
            if(issuerSerial==null){
                throw new NullPointerException("certDigest cannot be null.");
            }
            this.certDigest = certDigest;
            this.issuerSerial = issuerSerial;
            this.uri = uri;
        }
        
        public DOMDigestAlgAndValue getCertDigest() {
            return certDigest;
        }
        
        public DOMX509IssuerSerial getIssuerSerial() {
            return issuerSerial;
        }
        
        public String toString(){
            try{
                String dump = "";
                dump += "[CertDigest:\n" + this.getCertDigest().toString() + "]\n";
                dump += "[IssuerSerial:\n" + this.getIssuerSerial().toString() + "]\n";
                return dump;
            } catch(Exception e){
                e.printStackTrace();
                return "ERROR";
            }
        }
        
    }
}