/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;

/**DOM-based implementation of {@code CertificateValuesType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMCertificateValues extends DOMStructure{
    
    public final static String TAG_ENCAPSULATEDX509CERTIFICATE =
            "EncapsulatedX509Certificate";
    public final static String TAG_OTHERCERTIFICATE = "OtherCertificate";
    public final static String TAG_ID = "ID";
    
    /**certificates : byte[] or DOMOtherCertificate. May be empty but not null*/
    List<Object> certificates = new ArrayList<Object>();
    
    /**ID, optional (may be null, but not empty).*/
    String id;
    
    /** Creates a new instance of DOMCertificateValues
     *a copy of the list is made.
     *@param certs list of certificates. the certificate must be a
     * DOMEncapsulatedPKIDataType or DOMOtherCertificate.
     *@param id optional (may be null, but not empty);
     *@throws NullPointerExceptiion if certs is null
     *@throws IllegalArgumentException if X509 encoding fails, or if id is empty..
     */
    public DOMCertificateValues(List<Object> certs, String id) {
        if(certs==null){
            throw new NullPointerException("certs list is null");
        }
        Iterator it = certs.iterator();
        while(it.hasNext()){
            Object cert = it.next();
            if(cert instanceof byte[]
                    || cert instanceof DOMOtherCertificate){
                this.certificates.add(cert);
            }else{
                throw new IllegalArgumentException("Invalid class for certificate: "+
                        cert.getClass().getCanonicalName());
            }
        }
        
        if(id != null && id.length()==0){
            throw new IllegalArgumentException("id may be null but not empty.");
        }
        this.id = id;
        
    }
    public DOMCertificateValues(Element elem, DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        Element child = DOMUtils.getFirstChildElement(elem);
        while(child!=null){
            if(child.getLocalName().equals(TAG_ENCAPSULATEDX509CERTIFICATE)){
                DOMUtils.checkName(child, null, xadesNS);
                this.certificates.add(new DOMEncapsulatedPKIData(child, context));
            }
            else if(child.getLocalName().equals(TAG_OTHERCERTIFICATE)){
                DOMUtils.checkName(child, null, xadesNS);
                this.certificates.add(DOMOtherCertificate.unmarshall(child, context));
            }
            else{
                throw new MarshalException("invalid CertificateValuesType.");   
            }
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID); //may be null;
    }
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element certificateValuesElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        
        Iterator it=certificates.iterator();
        while(it.hasNext()){
            Object cert = it.next();
            if(cert instanceof DOMEncapsulatedPKIData){
                ((DOMEncapsulatedPKIData)cert).marshal(certificateValuesElem,
                        TAG_ENCAPSULATEDX509CERTIFICATE, context);
            }else if(cert instanceof DOMOtherCertificate){
                ((DOMOtherCertificate)cert).marshal(certificateValuesElem,
                       TAG_OTHERCERTIFICATE, context);
            }else{
                throw new MarshalException("Wrong class for a certificate: "+cert.getClass().getCanonicalName());
            }
        }
        DOMUtils.setAttributeID(certificateValuesElem, TAG_ID, id);
        parent.appendChild(certificateValuesElem);
    }

    /**Returns a list of DOMOtherCertificate or DOMEncapsulatedPKIDataType, never null
     * but may be empty.*/
    public List<Object> getCertificates() {
        return certificates;
    }

    /**Returns the ID.*/
    public String getId() {
        return id;
    }
    
    public String toString(){
        try{   
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            Iterator it = this.getCertificates().iterator();
            while(it.hasNext()){
                Object cert = it.next();
                String certType;
                if(cert instanceof DOMEncapsulatedPKIData){
                    certType = "EncapsulatedX509Certificate";
                }else{
                    certType = "OtherCertificate";
                }
                dump += "["+certType+"\n: "+cert.toString()+"]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
    
}
