/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;
import java.net.URI;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**DOM-based implementation of {@code CommitmentTypeIndicationType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMCommitmentTypeIndication extends DOMStructure{
    
    public final static String TAG_COMMITMENTTYPEID = "CommitmentTypeId";
    public final static String TAG_OBJECTREFERENCE = "ObjectReference";
    public final static String TAG_ALLSIGNEDDATAOBJECTS = "AllSignedDataObjects";
    public final static String TAG_COMMITMENTTYPEQUALIFIERS = "CommitmentTypeQualifiers";
    public final static String TAG_COMMITMENTTYPEQUALIFIER = "CommitmentTypeQualifier";
    
    /** Commitment type id. Never null.*/
    DOMObjectIdentifier commitmentTypeId;
    
    /** ObjectReferences. Never null, but may be empty.
     * whent the list is empty, 'AllSignedDataObjects' is selected.
     */
    List<URI> objectReferences = new ArrayList<URI>();
    
    /** CommitmentTypeQualifiers. Never null, but may be empty.*/
    List<DOMCommitmentTypeQualifier> commitmentTypeQualifiers =
            new ArrayList<DOMCommitmentTypeQualifier>();
    
    /**Creates a new instance of DOMCommitmentTypeIndication.
     *@param commitmentTypeId may not be null.
     *@param objectReferences must be null in case of 'AllSignedDataObject', 
     *but may not be empty.
     *@param commitmentTypeQualifiers may be null or empty.
     *@throws NullPointerException if commitmentTypeId is null
     *@throws InvalidArgumentException if objectReferences is an empty list.
     */
    public DOMCommitmentTypeIndication(
            DOMObjectIdentifier commitmentTypeId,
            List<URI> objectReferences,
            List<DOMCommitmentTypeQualifier> commitmentTypeQualifiers){
        if(commitmentTypeId==null){
            throw new NullPointerException("commitmentTypeId is null");
        }
        this.commitmentTypeId = commitmentTypeId;
        if(objectReferences!=null){
            if(objectReferences.size()==0){
                throw new IllegalArgumentException("objectReferences may not be empty." +
                        " Must be null in case of 'AllSignedDataObjects'");
            }
            this.objectReferences = objectReferences;
        }
        if(commitmentTypeQualifiers!=null){
            this.commitmentTypeQualifiers = commitmentTypeQualifiers;
        }
    }
    
    /** Creates a new instance of DOMCommitmentTypeIndication */
    public DOMCommitmentTypeIndication(Element elem, DOMCryptoContext context) 
    throws MarshalException{
        try{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        
        Element commitmentTypeIdElem = DOMUtils.getFirstChildElement(elem);
        DOMUtils.checkName(commitmentTypeIdElem, TAG_COMMITMENTTYPEID, xadesNS);
        commitmentTypeId = new DOMObjectIdentifier(commitmentTypeIdElem, context);
        
        Element nextElem = DOMUtils.getNextSiblingElement(commitmentTypeIdElem);
        while(nextElem.getLocalName().equals(TAG_OBJECTREFERENCE)){
            DOMUtils.checkName(nextElem, null, xadesNS);
            objectReferences.add(XSDAnyURI.parseURI(nextElem.getTextContent()));
            nextElem = DOMUtils.getNextSiblingElement(nextElem);
        }
        if(objectReferences.size()==0){
            DOMUtils.checkName(nextElem, TAG_ALLSIGNEDDATAOBJECTS, xadesNS);
            nextElem = DOMUtils.getNextSiblingElement(nextElem);
        }
        if(nextElem!=null){
            DOMUtils.checkName(nextElem, TAG_COMMITMENTTYPEQUALIFIERS, xadesNS);
            Element commitmentTypeQualifierElem = DOMUtils.
                    getFirstChildElement(nextElem);
            while(commitmentTypeQualifierElem!=null){
                DOMUtils.checkName(commitmentTypeQualifierElem, 
                        TAG_COMMITMENTTYPEQUALIFIER, xadesNS);
                commitmentTypeQualifiers.add(
                        DOMCommitmentTypeQualifier.unmarshall(
                        commitmentTypeQualifierElem, context));
            }
        }
        if(nextElem!=null){
            throw new MarshalException("Invalid CommitmentTypeIndication");
        }
        
        
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element commitmentTypeIndicationElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);

        commitmentTypeId.marshal(commitmentTypeIndicationElem,
                TAG_COMMITMENTTYPEID, context);
        
        if(!isAllSignedDataObjects()){
            Iterator<URI> it = objectReferences.iterator();
            while(it.hasNext()){
                Element objectReferenceElem = DOMUtils.createElement(
                doc, TAG_OBJECTREFERENCE, xadesNS, xadesPrefix);
                objectReferenceElem.appendChild(doc.createTextNode(
                        XSDAnyURI.printURI(it.next())));
                commitmentTypeIndicationElem.appendChild(objectReferenceElem);
            }
            
        }else{
            Element allSignedDataObjectsElem = DOMUtils.createElement(
                doc, TAG_ALLSIGNEDDATAOBJECTS, xadesNS, xadesPrefix);
            commitmentTypeIndicationElem.appendChild(allSignedDataObjectsElem);
        }
        
        if(commitmentTypeQualifiers.size()>0){
             Element commitmentTypeQualifiersElem = DOMUtils.createElement(
                doc, TAG_COMMITMENTTYPEQUALIFIERS, xadesNS, xadesPrefix);
             Iterator<DOMCommitmentTypeQualifier> it = commitmentTypeQualifiers.iterator();
             while(it.hasNext()){
                 it.next().marshal(commitmentTypeQualifiersElem,
                         TAG_COMMITMENTTYPEQUALIFIER, context);
             }
             commitmentTypeIndicationElem.appendChild(commitmentTypeQualifiersElem);
        }
        parent.appendChild(commitmentTypeIndicationElem);
    }

    /** returns true if the reference is 'AllSignedDataObject'.*/
    public boolean isAllSignedDataObjects(){
        if(objectReferences.size()<1){
            return true;
        }else{
            return false;
        }
    }

    /** Returns the commitment type ID. Never null.*/
    public DOMObjectIdentifier getCommitmentTypeId() {
        return commitmentTypeId;
    }

    /** Returns the commitment type qualifiers. May be empty (in case of
     * 'all signed data objects'), but never null.*/
    public List<DOMCommitmentTypeQualifier> getCommitmentTypeQualifiers() {
        return commitmentTypeQualifiers;
    }

    /**Returns the object references. May be empty, but never null.*/
    public List<URI> getObjectReferences() {
        return objectReferences;
    }
    public String toString(){
        try{   
            String dump = "";
            dump += "[CommitmentTypeId:\n"+this.getCommitmentTypeId() + "]\n";
            Iterator it = this.getObjectReferences().iterator();
            while(it.hasNext()){
                dump += "ObjectReference: "+it.next() + "\n";
            }
            if(this.isAllSignedDataObjects()){
                dump += "AllSignedDataObjects = true\n";
            }else{
                dump += "AllSignedDataObjects = false\n";
            }
            it = this.getCommitmentTypeQualifiers().iterator();
            while(it.hasNext()){
                dump += "[CommitmentTypeQualifiers:\n"+it.next()+"]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
