/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
  */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**DOM-based implementation of {@code CompleteRevocationRefs}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMCompleteRevocationRefs extends DOMStructure {
    
    public final static String TAG_CRLREFS = "CRLRefs";
    public final static String TAG_CRLREF = "CRLRef";
    public final static String TAG_OCSPREFS = "OCSPRefs";
    public final static String TAG_OCSPREF = "OCSPRef";
    public final static String TAG_OTHERREFS = "OtherRefs";
    public final static String TAG_OTHERREF = "OtherRef";
    public final static String TAG_ID = "Id";
    
    /**CRLRefs, may be empty but never null.*/
    List<DOMCRLRef> cRLRefs = new ArrayList<DOMCRLRef>();
    
    /**OCSPRefs, may be empty but never null.*/
    List<DOMOCSPRef> oCSPRefs = new ArrayList<DOMOCSPRef>();
    
    /**OtherRefs, may be empty but never null.*/
    List<DOMAnyType> otherRefs = new ArrayList<DOMAnyType>();
    
    /**Id, may be null, but not empty.*/
    String id;
    
    /** Creates a new instance of DOMCompleteRevocationRefs *
     *@param cRLRefs CRL references list, may be null or empty
     *@param oCSPRefs OCSP references list, may be null or empty
     *@param otherRefs Other references list (anyType), may be null or empty
     *@param id Id, optional (may be null, but not empty)
     *@throws IllegalArgumentException if id is empty.
     */
    public DOMCompleteRevocationRefs(
            List<DOMCRLRef> cRLRefs,
            List<DOMOCSPRef> oCSPRefs,
            List<DOMAnyType> otherRefs,
            String id) {
        if(cRLRefs!=null){this.cRLRefs = cRLRefs;}
        if(oCSPRefs!=null){this.oCSPRefs = oCSPRefs;}
        if(otherRefs!=null){this.otherRefs = otherRefs;}
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("id is empty");
        }
        this.id = id;
    }
    
    /**Unmarshal a CompleteRevocationRefs.*/
    public DOMCompleteRevocationRefs(Element elem, DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        Element child = DOMUtils.getFirstChildElement(elem);
        if(child!=null && child.getLocalName().equals(TAG_CRLREFS)){
            Element cRLRefElem = DOMUtils.getFirstChildElement(child);
            while(cRLRefElem !=null){
                DOMUtils.checkName(cRLRefElem, TAG_CRLREF, xadesNS);
                cRLRefs.add(new DOMCRLRef(cRLRefElem, context));
                cRLRefElem = DOMUtils.getNextSiblingElement(cRLRefElem);
            }
            if(cRLRefs.size()<1){
                throw new MarshalException("CRLRefs doesn't contain any element.");
            }
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_OCSPREFS)){
            Element oCSPRefElem = DOMUtils.getFirstChildElement(child);
            while(oCSPRefElem !=null){
                DOMUtils.checkName(oCSPRefElem, TAG_OCSPREF, xadesNS);
                oCSPRefs.add(new DOMOCSPRef(oCSPRefElem, context));
                oCSPRefElem = DOMUtils.getNextSiblingElement(oCSPRefElem);
            }
            if(oCSPRefs.size()<1){
                throw new MarshalException("OCSPRefs doesn't contain any element.");
            }
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_OTHERREFS)){
            Element otherRefElem = DOMUtils.getFirstChildElement(child);
            while(otherRefElem !=null){
                DOMUtils.checkName(otherRefElem, TAG_OTHERREF, xadesNS);
                otherRefs.add(new DOMAnyType(otherRefElem, context));
                otherRefElem = DOMUtils.getNextSiblingElement(otherRefElem);
            }
            if(otherRefs.size()<1){
                throw new MarshalException("OtherRefs doesn't contain any element.");
            }
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null){
            throw new MarshalException("Invalid CompleteRevocationRefs element.");
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID); //may be null;
        
    }
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element completeRevocationRefsElem = DOMUtils.createElement(doc, localName, 
                xadesNS, xadesPrefix);
        
        if(cRLRefs.size()>0){
            Element cRLRefsElem = DOMUtils.createElement(doc, TAG_CRLREFS, 
                xadesNS, xadesPrefix);
            Iterator<DOMCRLRef> it = cRLRefs.iterator();
            while(it.hasNext()){
                it.next().marshal(cRLRefsElem, TAG_CRLREF,
                        context);
            }
            completeRevocationRefsElem.appendChild(cRLRefsElem);
        }
          if(oCSPRefs.size()>0){
            Element oCSPRefsElem = DOMUtils.createElement(doc, TAG_OCSPREFS, 
                xadesNS, xadesPrefix);
            Iterator<DOMOCSPRef> it = oCSPRefs.iterator();
            while(it.hasNext()){
                it.next().marshal(oCSPRefsElem, TAG_OCSPREF,
                        context);
            }
            completeRevocationRefsElem.appendChild(oCSPRefsElem);
        }      
          if(otherRefs.size()>0){
            Element otherRefsElem = DOMUtils.createElement(doc, TAG_OTHERREFS, 
                xadesNS, xadesPrefix);
            Iterator<DOMAnyType> it = otherRefs.iterator();
            while(it.hasNext()){
                it.next().marshal(otherRefsElem, TAG_OTHERREF,
                        context);
            }
            completeRevocationRefsElem.appendChild(otherRefsElem);
        }      
        DOMUtils.setAttributeID(completeRevocationRefsElem, TAG_ID, this.id);
        parent.appendChild(completeRevocationRefsElem);
    }
}
