/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;
import java.net.URI;

/**DOM-based implementation of {@code DataObjectFormatType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMDataObjectFormat extends DOMStructure{
    
    public final static String TAG_DESCRIPTION = "Description";
    public final static String TAG_OBJECTIDENTIFIER = "ObjectIdentifier";
    public final static String TAG_MIMETYPE = "MimeType";
    public final static String TAG_ENCODING = "Encoding";
    public final static String TAG_OBJECTREFERENCE = "ObjectReference";
    
    /** Description. Never null, but may be empty.*/
    String description = new String();
    
    /** Object identifier. May be null.*/
    DOMObjectIdentifier objectIdentifier;
    
    /** Mime type. Never null, but may be empty.*/
    String mimeType = new String();
    
    /** Encoding. May be null.*/
    URI encoding;
    
    /** Object Reference attribute. Never null (required).*/
    URI objectReference;
    
    /** Creates a new instance of DOMDataObjectFormat.
     *@param description facultative (may be null or empty string).
     *@param objectIdentifier facultative (may be null).
     *@param mimeType facultative (may be null or empty string).
     *@param encoding facultative (may be null).
     *@param objectReference required (may not be null).
     *@thows NullPointerException if objectReference is null.
     */
    public DOMDataObjectFormat(
            String description,
            DOMObjectIdentifier objectIdentifier,
            String mimeType,
            URI encoding,
            URI objectReference) {
        if(description!=null){
            this.description = description;
        }
        this.objectIdentifier = objectIdentifier;
        if(mimeType!=null){
            this.mimeType = mimeType;
        }
        this.encoding = encoding;
        if(objectReference == null){
            throw new NullPointerException("Object Reference is required and" +
                    " may not be null.");
        }
        this.objectReference = objectReference;
    }
    
    public DOMDataObjectFormat(Element elem, DOMCryptoContext context)
    throws MarshalException{
        try{
            String xadesNS = XadESContext.getXadesNS(context);
            DOMUtils.checkName(elem, null, xadesNS);
            Element child = DOMUtils.getFirstChildElement(elem);
            
            if(child!=null && child.getLocalName().equals(TAG_DESCRIPTION)){
                DOMUtils.checkName(child, null, xadesNS);
                description = child.getTextContent();
                child = DOMUtils.getNextSiblingElement(child);
            }
            
            if(child!=null && child.getLocalName().equals(TAG_OBJECTIDENTIFIER)){
                DOMUtils.checkName(child, null, xadesNS);
                objectIdentifier = new DOMObjectIdentifier(child, context);
                child = DOMUtils.getNextSiblingElement(child);
            }
            
            if(child!=null && child.getLocalName().equals(TAG_MIMETYPE)){
                DOMUtils.checkName(child, null, xadesNS);
                mimeType = child.getTextContent();
                child = DOMUtils.getNextSiblingElement(child);
            }
            
            if(child!=null && child.getLocalName().equals(TAG_ENCODING)){
                DOMUtils.checkName(child, null, xadesNS);
                encoding = XSDAnyURI.parseURI(child.getTextContent());
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child!=null){
                throw new MarshalException("Invalid DataObjectFormat");
            }
            this.objectReference = XSDAnyURI.parseURI(elem.getAttributeNS(
                    null, TAG_OBJECTREFERENCE));
            if(objectReference==null){
                throw new MarshalException("ObjectReference attribute required.");
            }
            
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }

    
    public void marshal(Node parent, 
            String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element dataObjectFormatElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);

        if(description.length()>0){
            Element descriptionElem = DOMUtils.createElement(
                doc, TAG_DESCRIPTION, xadesNS, xadesPrefix);
            descriptionElem.appendChild(doc.createTextNode(description));
            dataObjectFormatElem.appendChild(descriptionElem);
        }
        if(objectIdentifier!=null){
            objectIdentifier.marshal(dataObjectFormatElem, 
                    TAG_OBJECTIDENTIFIER, context);
        }
        if(mimeType.length()>0){
             Element mimeTypeElem = DOMUtils.createElement(
                doc, TAG_MIMETYPE, xadesNS, xadesPrefix);
            mimeTypeElem.appendChild(doc.createTextNode(mimeType));
            dataObjectFormatElem.appendChild(mimeTypeElem);           
        }
        if(encoding!=null){
             Element encodingElem = DOMUtils.createElement(
                doc, TAG_ENCODING, xadesNS, xadesPrefix);
             encodingElem.appendChild(doc.createTextNode(
                     XSDAnyURI.printURI(encoding)));
             dataObjectFormatElem.appendChild(encodingElem);    
        }
        dataObjectFormatElem.setAttributeNS(null, TAG_OBJECTREFERENCE, 
                XSDAnyURI.printURI(objectReference));
        parent.appendChild(dataObjectFormatElem);
    }
    
    /**Returns the description. Never null but may be empty.*/
    public String getDescription() {
        return description;
    }

    /**Returns the encoding. May be null.*/
    public URI getEncoding() {
        return encoding;
    }

    /**Returns the mime type. Never null, but may be empty.*/
    public String getMimeType() {
        return mimeType;
    }

    /**Returns the object identifier. May be null.*/
    public DOMObjectIdentifier getObjectIdentifier() {
        return objectIdentifier;
    }

    public URI getObjectReference() {
        return objectReference;
    }

   public String toString(){
        try{   
            String dump = "";
            if(this.getDescription().length()>0){
                dump += " Description: "+this.getDescription() +"\n";
            }
            if(this.getObjectIdentifier()!=null){
                dump += "[ObjectIdentifier: \n"+this.getObjectIdentifier() + "]\n";
            }
            if(this.getMimeType().length()>0){
                dump += " MimeType: "+this.getMimeType()+"\n";
            }
            if(this.getEncoding()!=null){
                dump += " Encoding: "+this.getEncoding()+"\n";
            }
            dump += " ObjectReference: "+this.getObjectReference()+"\n";
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

    


}
