/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import java.net.URI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import be.cardon.xml.XSDBase64Binary;

/** DOM-based implementation of {@code EncapsulatedPKIDataType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMEncapsulatedPKIData extends DOMStructure{
    
    public static final String TAG_ENCODING = "Encoding";
    public static final String TAG_ID = "Id";
    
    /** http://uri.etsi.org/01903/v1.2.2#DER.*/
    public static final String DER_ENCODING_v122_URI =
            "http://uri.etsi.org/01903/v1.2.2#DER";
    
    /** http://uri.etsi.org/01903/v1.2.2#BER.*/
    public static final String BER_ENCODING_v122_URI =
            "http://uri.etsi.org/01903/v1.2.2#BER";
    
    /** http://uri.etsi.org/01903/v1.2.2#CER.*/
    public static final String CER_ENCODING_v122_URI =
            "http://uri.etsi.org/01903/v1.2.2#CER";
    
    /** http://uri.etsi.org/01903/v1.2.2#PER.*/
    public static final String PER_ENCODING_v122_URI =
            "http://uri.etsi.org/01903/v1.2.2#PER";
    
    /** http://uri.etsi.org/01903/v1.2.2#XER.*/
    public static final String XER_ENCODING_URI =
            "http://uri.etsi.org/01903/v1.2.2#XER";
    
    /**The (base64 decoded) binary content.*/
    byte[] encapsulatedPKIData;
    
    /**The optional ID. May be {@code null} but not empty.*/
    String id;
    
    /**The optional encoding URI. May be {@code null} but not empty.*/
    URI encoding;
    
    /** Create a new instance of {@code DOMEncapsulatedPKIData} *
     *@param encapsulatedPKIData the binary data (not base64 encoded
     *@param encoding the optional encoding (may be {@code null} or empty)
     *@param id the optional ID (can be {@code null} but not empty).
     *@throws NullPointerException if encapsulatedPKIData is {@code null}.
     *@throws IllegalArgumentException if id is empty.
     */
    public DOMEncapsulatedPKIData(
            byte[] encapsulatedPKIData,
            URI encoding,
            String id) {
        if(encapsulatedPKIData==null){
            throw new NullPointerException("encapsulatedPKIData may not be null.");
        }
        this.encapsulatedPKIData = encapsulatedPKIData;
        this.encoding = encoding;
        if(id != null && id.length()==0){
            throw new IllegalArgumentException("id may be null but not empty.");
        }
        this.id = id;
    }
    
    /**Unmarshal an element of type {@code DOMEncapsulatedPKIDataType}*/
    public DOMEncapsulatedPKIData(Element elem, DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        encapsulatedPKIData = XSDBase64Binary.decode(
                elem.getTextContent());
        String idString = elem.getAttributeNS(xadesNS,TAG_ID); //never null
        id = (idString.length()==0) ? null : idString;
        String encodingString = elem.getAttributeNS(
                xadesNS,TAG_ENCODING); //never null
        encoding = XSDAnyURI.parseURI(encodingString); //returns null if empty
    }
  
    public void marshal(Node parent, String localName,
            DOMCryptoContext context)
            throws MarshalException{
        marshal(parent, null, localName, context);
    }
    public void marshal(Node parent, Node nextSibling, String localName,
            DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element encapsPKIDataElem = DOMUtils.createElement(doc,
                localName, xadesNS, xadesPrefix);
        encapsPKIDataElem.appendChild(
                doc.createTextNode(XSDBase64Binary.encode(encapsulatedPKIData)));
        DOMUtils.setAttributeID(encapsPKIDataElem, TAG_ID, id);  //not writed if null
        if(encoding!=null){
            encapsPKIDataElem.setAttributeNS(null, TAG_ENCODING, encoding.toString());
        }
        parent.insertBefore(encapsPKIDataElem, nextSibling);
    }
    
    /**Returns the (decoded) encapsulated PKI data.*/
    public byte[] getEncapsulatedPKIData() {
        return encapsulatedPKIData;
    }
    
    /**Returns the encoding. May be null is not specified.*/
    public URI getEncoding() {
        return encoding;
    }
    
    /**Returns the Id (may be null if not specified).*/
    public String getId() {
        return id;
    }
    
    
    public String toString(){
        try{   
            String dump = "";
             if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            if(this.getEncoding()!=null){
                dump+= " attribute Encoding: "+this.getEncoding() +"\n";
            }
            dump += " Data (Base 64): "+ XSDBase64Binary.encode(this.getEncapsulatedPKIData())+"\n";
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

    
}
