/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMCanonicalizationMethod;
import java.net.URI;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.security.Provider;
import javax.xml.crypto.XMLCryptoContext;

/** DOM-based implementation of the abstract {@code GenericTimeStampType}.
 * for version 1.2.2 and 1.3.2
 *This object is a proxy element. You can modify an existing time stamp.
 *For example, you can add additional EncapsulatedTimeStamp.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public abstract class DOMGenericTimeStamp extends DOMStructure{
    
    public final static String TAG_INCLUDE = "Include";
    public final static String TAG_REFERENCEINFO = "ReferenceInfo";
    public final static String TAG_CANONICALIZATIONMETHOD = "CanonicalizationMethod";
    public final static String TAG_ENCAPSULATEDTIMESTAMP = "EncapsulatedTimeStamp";
    public final static String TAG_XMLTIMESTAMP = "XMLTimeStamp";
    public final static String TAG_ID = "Id";
    
    
     /**The TimeStamp element. Never null.
     */
    Element timeStampElem;
    

    /**The crypto context.*/
    DOMCryptoContext context;
    
     /** Creates a new instance of DOMGenericTimeStamp
     *@param id the optional Id
     *@param doc the DOM document
     *@param dsPrefix the prefix of the XMLSignature namespace
     *@param xadesPrefix the prefix of the XadES namespace
     *@param context the CryptoContext
     *@param provider the provider
     *@throws IllegalArgumentException if incorrect version.
     */
    public DOMGenericTimeStamp(String localname, 
            List<DOMInclude> includes,
            List<DOMReferenceInfo> refInfos,
            DOMCanonicalizationMethod canonicalizationMethod,
            List<DOMEncapsulatedPKIData> encapsulatedTimeStamps,
            List<DOMAnyType> xmlTimeStamps,
            String id, Document doc, DOMCryptoContext context)
            throws IllegalArgumentException, MarshalException{
       
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        if(!xadesNS.equals(Constants.Xades132SpecNS)){
            throw new IllegalArgumentException("Unvalid XadES version: "+xadesNS);
        }
        if(includes==null){
            includes = new ArrayList<DOMInclude>();
        }
        if(refInfos==null){
            refInfos = new ArrayList<DOMReferenceInfo>();
        }
        if(encapsulatedTimeStamps==null){
            encapsulatedTimeStamps = new ArrayList<DOMEncapsulatedPKIData>();
        }
        if(xmlTimeStamps==null){
            xmlTimeStamps = new ArrayList<DOMAnyType>();
        }
        if(includes.size()>0 && refInfos.size()>0){
            throw new IllegalArgumentException("Either includes, either refInfos must be null");
        }
        if(id!=null && id.length()==0){
            throw new IllegalArgumentException("ID cannot be empty (but may be null)");
        }
        
        
        timeStampElem = DOMUtils.createElement(
                doc, localname,
                xadesNS, xadesPrefix);
        

        this.context = context;
        
        this.setId(id);     

        Iterator<DOMInclude> itInc = includes.iterator();
        while(itInc.hasNext()){
            itInc.next().marshal(timeStampElem, TAG_INCLUDE, context);
        }
        
        Iterator<DOMReferenceInfo> itRefInfos = refInfos.iterator();
        while(itRefInfos.hasNext()){
            itRefInfos.next().marshal(timeStampElem, TAG_REFERENCEINFO, context);
        }
        if(canonicalizationMethod!=null){
            canonicalizationMethod.marshal(timeStampElem, dsPrefix, context);
        }
        Iterator<DOMEncapsulatedPKIData> itEncaps = encapsulatedTimeStamps.iterator();
        while(itEncaps.hasNext()){
            DOMEncapsulatedPKIData encapsulatedTimeStamp = itEncaps.next();
            encapsulatedTimeStamp.marshal(timeStampElem, 
                    TAG_ENCAPSULATEDTIMESTAMP,context);
        }
        Iterator<DOMAnyType> itXML = xmlTimeStamps.iterator();
        while(itXML.hasNext()){
            DOMAnyType xmlTimeStamp = itXML.next();
            xmlTimeStamp.marshal(timeStampElem,
                    TAG_XMLTIMESTAMP, context);
        }
    }
     /**Unmarshal a timestamp.
     *@param elem the element to unmarshall
     *@param context the cryptoContext
     *@param provider the provider
     *@param dsPrefix the prefix of the XMLSignature namepsace
     *@param xadesPrefix the prefix of the XadES namespace
     */
    public DOMGenericTimeStamp(Element elem,
            DOMCryptoContext context)
    throws MarshalException{
        this.timeStampElem = elem;
        this.context = context;
                String xadesNS = XadESContext.getXadesNS(context);
          //structure check
        try{
            DOMUtils.checkName(elem, null, xadesNS);
            int includeSize = 0;
            int refInfosSize = 0;
            Element child = DOMUtils.getFirstChildElement(elem);
            while(child.getLocalName().equals(TAG_INCLUDE)){
                DOMUtils.checkName(child, null, xadesNS);
                includeSize++;
                child = DOMUtils.getNextSiblingElement(child);
            }
            while(child.getLocalName().equals(TAG_REFERENCEINFO)){
                DOMUtils.checkName(child, null, xadesNS);
                refInfosSize++;
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(includeSize>0 && refInfosSize>0){
                throw new MarshalException("either Include, either " +
                        "ReferenceInfo must be absent");
            }
            if(child.getLocalName().equals(TAG_CANONICALIZATIONMETHOD)){
                DOMUtils.checkName(child, null, XMLSignature.XMLNS);
                child = DOMUtils.getNextSiblingElement(child);
            }
            
            if(child.getLocalName().equals(TAG_ENCAPSULATEDTIMESTAMP)){
                DOMUtils.checkName(child, null, xadesNS);
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child !=null && child.getLocalName().equals(TAG_XMLTIMESTAMP)){
                DOMUtils.checkName(child, null, xadesNS);
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child !=null){
                throw new MarshalException("unknown element: "+child.getLocalName());
            }
            
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
        
    }
    
    /**Set the Id. Not empty (may be null).*/
    public void setId(String id){
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may not be null (but may be empty.");
        }
        DOMUtils.setAttributeID(this.timeStampElem, TAG_ID, id);
    }
    
    /**Returns the Id, or null if not specified.*/
    public String getId(){
        return DOMUtils.getAttributeValue(this.timeStampElem, TAG_ID);
    }
         
    /**Returns the includes.*/
    public List<DOMInclude> getIncludes() throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        NodeList list = timeStampElem.getElementsByTagNameNS(xadesNS, TAG_INCLUDE);
        List<DOMInclude> includes = new ArrayList<DOMInclude>();
        for(int i=0;i<list.getLength();i++){
            includes.add(new DOMInclude((Element)list.item(i), context));
        }
        return includes;
    }
 
    /**Returns the canonicalization method or null if not specified.*/
    public DOMCanonicalizationMethod getCanonicalizationMethod() throws MarshalException{
        NodeList list = timeStampElem.getElementsByTagNameNS(
                XMLSignature.XMLNS, TAG_CANONICALIZATIONMETHOD);
        Element canonicElem = (Element)list.item(0);
        if(canonicElem != null){
            return new DOMCanonicalizationMethod(canonicElem, context, 
                    XadESContext.getXMLSignatureProvider(context));
        }else{
            return null;
        }
    }
    
    /**Returns the referenceInfos.*/
    public List<DOMReferenceInfo> getReferenceInfos() throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        NodeList list = timeStampElem.getElementsByTagNameNS(xadesNS, TAG_REFERENCEINFO);
        List<DOMReferenceInfo> refInfos = new ArrayList<DOMReferenceInfo>();
        for(int i=0;i<list.getLength();i++){
            refInfos.add(new DOMReferenceInfo((Element)list.item(i), context));
        }
        return refInfos;
    }
    
    /**Returns the encapsulatedTimeStamps.*/
    public List<DOMEncapsulatedPKIData> getEncapsulatedTimeStamps()
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        NodeList list = timeStampElem.getElementsByTagNameNS(xadesNS, 
                TAG_ENCAPSULATEDTIMESTAMP);
        List<DOMEncapsulatedPKIData> encapsTSs = 
                new ArrayList<DOMEncapsulatedPKIData>();
        for(int i=0;i<list.getLength();i++){
            encapsTSs.add(new DOMEncapsulatedPKIData((Element)list.item(i), context));
        }
        return encapsTSs;
    }
    
    /**Returns the XMLTimeStamps.*/
    public List<DOMAnyType> getXMLTimeStamps()
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        NodeList list = timeStampElem.getElementsByTagNameNS(xadesNS, 
                TAG_XMLTIMESTAMP);
        List<DOMAnyType> xmlTSs = 
                new ArrayList<DOMAnyType>();
        for(int i=0;i<list.getLength();i++){
            xmlTSs.add(new DOMAnyType((Element)list.item(i), context));
        }
        return xmlTSs;
    }    
    
     /**Add an encapsulated time stamp.*/
    public void addEncapsulatedTimeStamp(DOMEncapsulatedPKIData encapsTS) 
        throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        Node nextSibling;
        NodeList list = timeStampElem.getElementsByTagNameNS(xadesNS, 
                TAG_XMLTIMESTAMP); 
        nextSibling = list.item(0); //return null if empty list
        encapsTS.marshal(timeStampElem, nextSibling, 
                TAG_ENCAPSULATEDTIMESTAMP, context);
        
    }
       
    /**Add an XML time stamp.*/
    public void addXMLTimeStamp(DOMAnyType xmlTimeStamp) 
        throws MarshalException{
        xmlTimeStamp.marshal(timeStampElem, 
                TAG_ENCAPSULATEDTIMESTAMP, context);
    }

    public void marshal(Node parent, Node nextSibling)throws MarshalException{
        parent.insertBefore(timeStampElem, nextSibling);
    }
    
    public String toString(){
        try{   
            String dump = "";
             if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            Iterator it = this.getIncludes().iterator();
            while(it.hasNext()){
                dump += "[Include:\n" + it.next() + "]\n";
            }
            it = this.getReferenceInfos().iterator();
            while(it.hasNext()){
                dump += "[ReferenceInfo:\n" + it.next() + "]\n";
            }
            DOMCanonicalizationMethod canon = this.getCanonicalizationMethod();
            if(canon!=null){
                dump += "[CanonicalizationMethod:\n"+canon+"]\n";
            }
            it = this.getEncapsulatedTimeStamps().iterator();
            while(it.hasNext()){
                dump += "[EncapsulatedTimeStamp:\n" + it.next() + "]\n";
            }
            it = this.getXMLTimeStamps().iterator();
            while(it.hasNext()){
                dump += "[XMLTimeStamps:\n" + it.next() + "]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

    
    
}

