/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import be.cardon.xml.XSDBase64Binary;
import be.cardon.xml.XSDDateTime;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.util.Date;
import java.util.GregorianCalendar;
import java.net.URI;

/** DOM-based implementation of {@code OCSPRefType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMOCSPRef extends DOMStructure{
    
    public final static String TAG_OCSPIDENTIFIER = "OCSPIdentifier";
    public final static String TAG_DIGESTALGANDVALUE = "DigestAlgAndValue";
    public final static String TAG_BYNAME = "ByName";
    public final static String TAG_BYKEY = "ByKey";
    public final static String TAG_RESPONDERID = "ResponderID";
    public final static String TAG_PRODUCEDAT = "ProducedAt";
    public final static String TAG_URI = "URI";
    
    /**OCSPIdentifier, never null.*/
    OCSPIdentifier oCSPIdentifier;
    
    /**DigestAlgAndValue, may be null.*/
    DOMDigestAlgAndValue digestAlgAndValue;
    
    /** Creates a new instance of DOMOCSPRef.*
     *@param oCSPIdentifier, never null
     *@param digestAlgAndValue, may be null.
     *@throws NullPointerException if oCSPIdentifier is null.
     */
    public DOMOCSPRef(
            OCSPIdentifier oCSPIdentifier,
            DOMDigestAlgAndValue digestAlgAndValue) {
        if(oCSPIdentifier==null){
            throw new NullPointerException("oCSPIdentifier is null.");
        }
        this.oCSPIdentifier = oCSPIdentifier;
        this.digestAlgAndValue = digestAlgAndValue;
    }
    
    /** Unmarshal a OCSPRef.*/
    public DOMOCSPRef(Element elem, DOMCryptoContext context)throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        
        Element oCSPIdentifierElem = DOMUtils.getFirstChildElement(elem);
        DOMUtils.checkName(oCSPIdentifierElem, TAG_OCSPIDENTIFIER, xadesNS);
        
        Element responderIDElem = DOMUtils.getFirstChildElement(oCSPIdentifierElem);
        DOMUtils.checkName(responderIDElem, TAG_RESPONDERID, xadesNS);
        Element child = DOMUtils.getFirstChildElement(responderIDElem);
        Object responderID;
        if(child.getLocalName().equals(TAG_BYNAME)){
            DOMUtils.checkName(child, null, xadesNS);
            responderID = child.getTextContent();
        }else if(child.getLocalName().equals(TAG_BYKEY)){
            DOMUtils.checkName(child, null, xadesNS);
            responderID = XSDBase64Binary.decode(child.getTextContent());
        }else{
            throw new MarshalException("invalid ResponderID.");
        }
        Element producedAtElem = DOMUtils.getNextSiblingElement(responderIDElem);
        DOMUtils.checkName(producedAtElem, TAG_PRODUCEDAT, xadesNS);
        GregorianCalendar producedAt = XSDDateTime.parseDateTime(
                producedAtElem.getTextContent()).toGregorianCalendar();
        URI uri = XSDAnyURI.parseURI(oCSPIdentifierElem.getAttributeNS(
                null, TAG_URI));
        
        oCSPIdentifier = new OCSPIdentifier(responderID, producedAt, uri);
        
        Element digestAlgAndValueElem = DOMUtils.getNextSiblingElement(
                oCSPIdentifierElem);
        if(digestAlgAndValueElem!=null){
            DOMUtils.checkName(digestAlgAndValueElem, TAG_DIGESTALGANDVALUE,
                    xadesNS);
            this.digestAlgAndValue = new DOMDigestAlgAndValue(digestAlgAndValueElem,
                    context);
        }
    }
    
   public void marshal(Node parent, 
            String localName,  DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element oCSPRefElem = DOMUtils.createElement(doc,
                localName, xadesNS, xadesPrefix);
        
        Element oCSPIdentifierElem = DOMUtils.createElement(doc,
                TAG_OCSPIDENTIFIER, xadesNS, xadesPrefix);
        Element responderIDElem = DOMUtils.createElement(doc,
                TAG_RESPONDERID, xadesNS, xadesPrefix);
        if(oCSPIdentifier.getResponderID() instanceof String){
            Element byNameElem = DOMUtils.createElement(doc,
                TAG_BYNAME, xadesNS, xadesPrefix);
            byNameElem.appendChild(doc.createTextNode(
                    (String)oCSPIdentifier.getResponderID()));
            responderIDElem.appendChild(byNameElem);
        }else{
             Element byKeyElem = DOMUtils.createElement(doc,
                TAG_BYKEY, xadesNS, xadesPrefix);
            byKeyElem.appendChild(doc.createTextNode(
                    XSDBase64Binary.encode(
                    (byte[])oCSPIdentifier.getResponderID())));
            responderIDElem.appendChild(byKeyElem);           
        }
        oCSPIdentifierElem.appendChild(responderIDElem);
        
        Element producedAtElem = DOMUtils.createElement(doc,
                TAG_PRODUCEDAT, xadesNS, xadesPrefix);
        producedAtElem.appendChild(doc.createTextNode(
                XSDDateTime.printDateTime(oCSPIdentifier.getProducedAt())));
        oCSPIdentifierElem.appendChild(producedAtElem);
        
        if(oCSPIdentifier.getUri()!=null){
            oCSPIdentifierElem.setAttributeNS(null, TAG_URI,
                    XSDAnyURI.printURI(oCSPIdentifier.getUri()));
        }
        oCSPRefElem.appendChild(oCSPIdentifierElem);
        
        if(digestAlgAndValue!=null){
            digestAlgAndValue.marshal(oCSPRefElem, 
                TAG_DIGESTALGANDVALUE, context);
        }
        parent.appendChild(oCSPRefElem);
        
   }
    
    public class OCSPIdentifier{
        
        /**ResponderID, must be a String or a byte[], never null.*/
        Object responderID;
        
        /**ProducetAt, never null.*/
        GregorianCalendar producedAt;
        
        /**URI, may be null.*/
        URI uri;
        
        /**Constructor.
         *@param responderID, must be a String or a byte[], required (not null, not empty)
         *@param producedAt, required (not null)
         *@param uri, optional (may be null)
         *@throws NullPointerException if responderID or producedAt is null
         *@throws IllegalArgumentException if responderID is empty.
         */
        public OCSPIdentifier(Object responderID,
                GregorianCalendar producedAt,
                URI uri){
            if(responderID==null) {
                throw new NullPointerException("ResponderID is null.");
            }
            if(producedAt==null) {
                throw new NullPointerException("ProducedAt is null.");
            }
            if(responderID instanceof String){
                if(((String)responderID).length()<1){
                    throw new IllegalArgumentException("ResponderID is empty String.");
                }
            }else if(!(responderID instanceof byte[])){
                throw new IllegalArgumentException("ResponderID must be a byte[] or String.");
            }
            this.responderID = responderID;
            this.producedAt = producedAt;
            this.uri = uri;
        }
        
        public GregorianCalendar getProducedAt() {
            return producedAt;
        }
        
        /**Returns responderID. The object is a String (by name) or
         * a byte[]. Use 'instanceof' to determine the class.*/
        public Object getResponderID(){
            return responderID;
        }
        
        public URI getUri() {
            return uri;
        }
        
        
        
    }
    
}
