/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import java.net.URI;
import java.util.List;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;

/**DOM-based implementation of {@code ObjectIdentifierType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMObjectIdentifier extends DOMStructure{
    
    /**Qualifier {@code OIDAsURI) value.*/
    public final static String OIDAsURI = "OIDAsURI";
    
    /**Qualifier {@code OIDAsURN} value.*/
    public final static String OIDAsURN = "OIDAsURN";
    
    public final static String TAG_IDENTIFIER = "Identifier";
    public final static String TAG_DESCRIPTION = "Description";
    public final static String TAG_DOCUMENTATIONREFERENCES = "DocumentationReferences";
    public final static String TAG_QUALIFIER = "Qualifier";
    public final static String TAG_DOCUMENTATIONREFERENCE = "DocumentationReference";
    
    /**The {@code Identifier} element.*/
    URI identifier;
    
    /**The optional {@code Qualifier} attribute of the {@code Identifier} element.*
     *May be OIDAsURI, OIDAsURN or {@code null}.
     */
    String qualifier;
    
    /**The facultative {@code description} element.
     * May be an empty string, but not null*/
    String description = new String();
    
    /**The facultative {@code DocumentationReferences} element.
     *May be an empty list, but never null.
     */
    List<URI> documentationReferences = new java.util.ArrayList<URI>();
    
    /** Creates a new {@code ObjectIdentifier}.
     *@param identifier the identifying URI.
     *@param qualifier the optional qualifier attribute (must be 
     * {@code OIDAsURI}, {@code OIDAsURN} or {@code null}).
     *@param description the optional informal text describing the object identifier.
     *@param documentationReferences an arbitraty number of references pointing
     * to the further explanatory documentation of the object identifier.
     *@throws NullPointerException if identifier is {@code null}.
     *@throws IllegalArgumentException if invalid qualifier is given.
     */
    public DOMObjectIdentifier(
            URI identifier,
            String qualifier,
            String description,
            List<URI> documentationReferences)
            
            {
        
        if(identifier==null){
            throw new NullPointerException("identifier may not be null.");
        }
        this.identifier = identifier;
        
        if(qualifier!=null){
            if(qualifier.equals(OIDAsURI) || qualifier.equals(OIDAsURN)){
                this.qualifier = qualifier;
            }else{
                throw new IllegalArgumentException("Invalid qualifier. Must be OIDAsURI or OIDAsURN");
            }
        }else{
            this.qualifier=null;
        }
        this.description = (description==null) ? new String() : description;
        this.documentationReferences = (documentationReferences==null) ?
            new java.util.ArrayList<URI>() : documentationReferences;
    }
    
    /**Unmarshal a {@code ObjectIdentifier} element.
     *@param elem the {@code ObjectIdentifier} element.
     *@param xadesNS, the expected xades namespace.
     *@thorws MarshalException
     */
    public DOMObjectIdentifier(Element elem, DOMCryptoContext context)
    throws MarshalException{
        
        try{
            String xadesNS = XadESContext.getXadesNS(context);
            DOMUtils.checkName(elem, null, xadesNS);
            
            Element identifierElem = DOMUtils.getFirstChildElement(elem);
            DOMUtils.checkName(identifierElem, TAG_IDENTIFIER, xadesNS);
            identifier = XSDAnyURI.parseURI(identifierElem.getTextContent());
             //getTextContent never returns null
            String qualifierString = identifierElem.getAttribute(TAG_QUALIFIER); 
             //getAttribute never returns null
            this.qualifier = (qualifierString.length()==0) ? null : qualifierString;
            
            Element child = DOMUtils.getNextSiblingElement(identifierElem);
            if(child!=null && child.getLocalName().equals(TAG_DESCRIPTION)){
                DOMUtils.checkName(child, null, xadesNS);
                description = child.getTextContent(); // this method never returns null
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child!=null && child.getLocalName().equals(TAG_DOCUMENTATIONREFERENCES)){
                DOMUtils.checkName(child, null, xadesNS);
                Element refElem = DOMUtils.getFirstChildElement(child);
                while(refElem !=null){
                    DOMUtils.checkName(refElem, TAG_DOCUMENTATIONREFERENCE, xadesNS);
                    documentationReferences.add(XSDAnyURI.parseURI(refElem.getTextContent()));
                    refElem = DOMUtils.getNextSiblingElement(refElem);
                }
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child!=null){
                throw new MarshalException("unexpected element: "+child.getLocalName());
            }
            
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }

    public void marshal(Node parent, String localName, 
            DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element oidElem = DOMUtils.createElement(doc, 
                localName, xadesNS, xadesPrefix);
        
        Element identifierElem = DOMUtils.createElement(doc,
                TAG_IDENTIFIER, xadesNS, xadesPrefix);
        identifierElem.appendChild(doc.createTextNode(identifier.toString()));
        if(qualifier!=null){
            if(qualifier!=OIDAsURI && qualifier!=OIDAsURN){
                throw new MarshalException("invalid qualifier:"+qualifier);
            }
            identifierElem.setAttributeNS(null, TAG_QUALIFIER, qualifier);
        }
        oidElem.appendChild(identifierElem);
        
        if(description.length()>0){
            Element descriptionElem = DOMUtils.createElement(doc,
                    TAG_DESCRIPTION, xadesNS, xadesPrefix);
            descriptionElem.appendChild(doc.createTextNode(description));
            oidElem.appendChild(descriptionElem);
        }
        
        if(documentationReferences.size()>0){
            Element docRefsElem = DOMUtils.createElement(doc,
                    TAG_DOCUMENTATIONREFERENCES, xadesNS, xadesPrefix);
            Iterator<URI> it = documentationReferences.iterator();
            while(it.hasNext()){
                Element docRefElem = DOMUtils.createElement(doc,
                        TAG_DOCUMENTATIONREFERENCE, xadesNS, xadesPrefix);
                docRefElem.appendChild(doc.createTextNode(
                        XSDAnyURI.printURI(it.next())));
                docRefsElem.appendChild(docRefElem);
            }
            oidElem.appendChild(docRefsElem);
        }
        
        parent.appendChild(oidElem);
    }        

    /**Returns the description. Never {@code null}, but may be empty.*/
    public String getDescription() {
        return description;
    }

    /**Returns the documentation references. Never {@code null}, but may be empty.*/
    public List<URI> getDocumentationReferences() {
        return documentationReferences;
    }

    /**Returns the identifier. Never {@code null}.*/
    public URI getIdentifier() {
        return identifier;
    }

    /**Returns the qualifier or {@code null} is no qualifier is defined.*/
    public String getQualifier() {
        return qualifier;
    }
    
    public String toString(){
        try{   
            String dump="";
            dump += " Identifier: "+this.getIdentifier()+"\n";
            String qualifier = this.getQualifier();
            if(qualifier!=null){
                dump += " Qualifier: "+ qualifier+"\n";
            }
            dump += " Description: " +this.getDescription()+"\n"; //may be empty
            Iterator it = this.getDocumentationReferences().iterator();
            while(it.hasNext()){
                dump += " DocumentationReference: " + it.next() + "\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

}
