/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMXMLObject;
import be.cardon.dsig.dom.DOMXMLSignature;
import be.cardon.xml.XSDAnyURI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.XMLCryptoContext;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.xml.crypto.dsig.XMLSignature;
import java.security.Provider;
import java.net.URI;

/**DOM-based implementation of {@code QualifyingProperties}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMQualifyingProperties {
    
    public static String TAG_QUALIFYINGPROPERTIES = "QualifyingProperties";
    public static String TAG_SIGNEDPROPERTIES = "SignedProperties";
    public static String TAG_UNSIGNEDPROPERTIES = "UnsignedProperties";
    public static String TAG_TARGET = "Target";
    public static String TAG_ID = "Id";
    
    /**The qualifying properties element.*/
    Element QPElem;
    
    /**The cryptoContext.*/
    DOMCryptoContext context;
       
    /**Creates a new instance of QualifiedProp.*/
    public DOMQualifyingProperties(Document doc,
            URI target,
            XMLCryptoContext ctxt)throws MarshalException {
        
        DOMCryptoContext context = (DOMCryptoContext) ctxt;
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        //check xadesVersion
        if(target==null){
            throw new NullPointerException("Target is null.");
        }
        if(!isSupportedXadesNS(xadesNS)){
            throw new IllegalArgumentException("This version of the Qualifying properties is not " +
                    "supported yet. XadES namespace:" + xadesNS);
        }
        if(context==null){
            throw new NullPointerException("context is null");
        }
        this.QPElem = DOMUtils.createElement(doc, TAG_QUALIFYINGPROPERTIES, xadesNS,xadesPrefix);
        this.QPElem.setAttributeNS(null, TAG_TARGET, XSDAnyURI.printURI(target));
        
        //add xmlns:ds attribute
        if(dsPrefix!=null){
            this.QPElem.setAttribute("xmlns:"+dsPrefix, XMLSignature.XMLNS);
        }
        this.context = (DOMCryptoContext)context;
        
    }
    
    /** Creates a new instance from an existing DOM element.
     *@param qualifPropElem the qualified Properties Element.
     */
    public DOMQualifyingProperties(Element qualifPropElem,
            DOMCryptoContext context) throws
            MarshalException{
        this.QPElem = qualifPropElem;
        this.context = (DOMCryptoContext)context;
        
        //check the local name
        if(!QPElem.getLocalName().equals(TAG_QUALIFYINGPROPERTIES)){
            throw new MarshalException("The given element is not a QualifiyngProperties.");
        }
        
        //check the namespace (version)
        String xadesNS = getXadesNS();
        if(!isSupportedXadesNS(getXadesNS())){
            throw new MarshalException("The version of the Qualifying properties is not " +
                    "supported yet. XadES namespace:"+ getXadesNS());
        }
        
        XadESContext.setXadesNS(context, xadesNS);
        
        //check that there is a non empty target.
        if(getTarget()==null){
            throw new MarshalException("The Target attribute is not defined.");
        }
    }
    
    /**This static function return true for a supported namespace.*/
    public static boolean isSupportedXadesNS(String xadesNS){
        if(xadesNS.equals(Constants.Xades132SpecNS)){
            return true;
        }else{
            return false;
        }
    }
    /**Never null.
     */
    public URI getTarget() throws MarshalException{
        return XSDAnyURI.parseURI(QPElem.getAttributeNS(null, TAG_TARGET));
    }
    
    /**Returns the Id, may be null.*/
    public String getId(){
        return DOMUtils.getAttributeValue(QPElem, TAG_ID);
    }
    
    /**Returns the owner document.*/
    public Document getDocument(){
        return DOMUtils.getOwnerDocument(QPElem);
    }
    
    /**Set the Id, may be null.*/
    public void setId(String id){
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may be null but not empty.");
        }
        DOMUtils.setAttributeID(QPElem, TAG_ID, id);
    }
    
    /**Returns the signed properties, or null if not specified.*/
    public DOMSignedProperties getSignedProperties()
    throws MarshalException{
        Element signedPropElem = (Element)QPElem.getElementsByTagNameNS(
                getXadesNS(), TAG_SIGNEDPROPERTIES).item(0);
        if(signedPropElem==null){
            return null;
        }
        return new DOMSignedProperties(signedPropElem, context);
    }
    /**Returns the unsigned properties, or null if not specified.*/
    public DOMUnsignedProperties getUnsignedProperties()
    throws MarshalException{
        Element unsignedPropElem = (Element)QPElem.getElementsByTagNameNS(
                getXadesNS(), TAG_UNSIGNEDPROPERTIES).item(0);
        if(unsignedPropElem==null){
            return null;
        }
        return new DOMUnsignedProperties(unsignedPropElem, context);
    }
    
    /**Set the signed properties. Overwrite the old signed properties if any.*/
    public void setSignedProperties(DOMSignedProperties signedProps)
    throws MarshalException{
        Element signedPropElem = (Element)QPElem.getElementsByTagNameNS(
                getXadesNS(), TAG_SIGNEDPROPERTIES).item(0);
        if(signedPropElem!=null){
            QPElem.removeChild(signedPropElem);
        }
        //the marshal function of DOMSignedProperties add the child as first child.
        signedProps.marshal(QPElem, TAG_SIGNEDPROPERTIES, context);
    }
    
    /**Set the unsigned properties. Overwrite the old unsigned properties if any.*/
    public void setUnsignedProperties(DOMUnsignedProperties unsignedProps)
    throws MarshalException{
        Element unsignedPropElem = (Element)QPElem.getElementsByTagNameNS(
                getXadesNS(), TAG_UNSIGNEDPROPERTIES).item(0);
        if(unsignedPropElem!=null){
            QPElem.removeChild(unsignedPropElem);
        }
        QPElem.appendChild(unsignedProps.UPElem);
    }
    
    /**Get the unsigned signature, or create a new one.*/
    public DOMUnsignedProperties getOrCreateUnsignedSignature()
    throws MarshalException{
        DOMUnsignedProperties up = getUnsignedProperties();
        if(up==null){
            up = new DOMUnsignedProperties(null,context);
        }
        return up;
    }
    
    /**Retrieves the xades namespace from this QualifyingProperties element.*/
    public String getXadesNS(){
        return QPElem.getNamespaceURI();
    }
    
    /**Retrievers the xades prefix from this QualifyingProperties element.*/
    public String getXadesPrefix(){
        return QPElem.getPrefix();
    }
    
    /**Returns the qualifying properties (with "direct incorporation" as defined
     * in ETSI TS 101 903 V1.3.2 paragraph 6.3) of a given XMLSignature (not the
     *qualifying properties referenced by a QualifyingPropertiesReference.*
     *Returns null if not specified.
     */
    public static DOMQualifyingProperties getQualifyingProperties(XMLSignature sig,
           DOMCryptoContext context)
            throws MarshalException{
        List<DOMXMLObject> objs = (List<DOMXMLObject>) sig.getObjects();
        Iterator<DOMXMLObject> it = objs.iterator();
        while(it.hasNext()){
            DOMXMLObject obj = it.next();
            List<javax.xml.crypto.dom.DOMStructure> xmlstructs =
                    (List<javax.xml.crypto.dom.DOMStructure>)obj.getContent();
            Iterator<javax.xml.crypto.dom.DOMStructure> itStruct = xmlstructs.iterator();
            while(itStruct.hasNext()){
                Node node = itStruct.next().getNode();
                if(node.getNodeType()==Node.ELEMENT_NODE){
                    Element elem = (Element)node;
                    if(elem.getLocalName().equals(TAG_QUALIFYINGPROPERTIES)){
                        return new DOMQualifyingProperties(elem,
                                context);
                    }
                }
                
            }
            
            
        }
        return null;
    }
    
    /**Returns the qualifying properties references of a given XMLSignature.
     *Returns null if not specified.
     */
    //public static DOMQualifiedPropertiesReferences getQualifyingPropertiesReferences(XMLSignature sig){
    
    //}
    /**Returns the element.*/
    
    public Element getQPElem() {
        return QPElem;
    }
    
    public String toString(){
        try{   
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            dump += "target="+this.getTarget().toString()+"\n";
            if(this.getSignedProperties()!=null){
                dump += "[SignedProperties:\n" + this.getSignedProperties().toString() 
                + "] end SignedProperties\n";
            }
            if(this.getUnsignedProperties()!=null){
                dump += "[UnsignedProperties:\n" + this.getUnsignedProperties().toString() 
                + "] end UnsignedProperties\n";
            }
            return dump;
        } catch(MarshalException e){
            e.printStackTrace();
            return "Marshal ERROR";
        }
    }
    
    
    
}
