/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMXMLObject;
import be.cardon.dsig.dom.DOMXMLSignature;
import be.cardon.xml.XSDAnyURI;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.XMLCryptoContext;
import java.security.Provider;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.net.URI;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**DOM-based implementation of {@code QualifyingPropertiesReference}.
 * Proxy element
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMQualifyingPropertiesReference {
    
    public final static String TAG_QUALIFYINGPROPERTIESREFERENCE = 
            "QualifyingPropertiesReference";
    public final static String TAG_URI = 
            "URI";
    public final static String TAG_ID = "Id";
    
    /**The <QualifyingPropertiesReference> element.*/
    Element QPRElem;
       
    /**The crypto context.*/
    DOMCryptoContext context;
        
    /** Creates a new instance of <QualifyingPropertiesReference>.
     *@throws NullPointerException if the URI is null.
     *@throws IllegalArgumentException if the id is Empty (may be null).
     */
    public DOMQualifyingPropertiesReference(URI uri, String id, Document doc,
            DOMCryptoContext context)
            throws NullPointerException, IllegalArgumentException,
             MarshalException{

        if(uri==null){
            throw new NullPointerException("the URI is null.");
        }
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        QPRElem = DOMUtils.createElement(
                doc, TAG_QUALIFYINGPROPERTIESREFERENCE,
                xadesNS, xadesPrefix);

        this.context = context;

        
        this.setId(id);
        QPRElem.setAttributeNS(null, TAG_URI, XSDAnyURI.printURI(uri));
    }
    
    /**Unmarshal a QualifyingPropertiesReference.
     *@param elem the element to unmarshall
     *@param xadesNS the XadES namespace
     *@param context the cryptoContext
     *@param xadesPrefix the prefix of the XadES namespace
     */
    public DOMQualifyingPropertiesReference(Element elem,
            DOMCryptoContext context)
            throws MarshalException{
        this.QPRElem = elem;
        this.context = context;
        String xadesNS = getXadesNS();
        XadESContext.setXadesNS(context, xadesNS);
        
        DOMUtils.checkName(elem, TAG_QUALIFYINGPROPERTIESREFERENCE, xadesNS);
        
        //check structure
        if(XSDAnyURI.parseURI(elem.getAttributeNS(null, TAG_URI))==null){
            throw new MarshalException("URI attribute not found");
        }
    }
  
    /**Retrieves the xades namespace from this QualifyingPropertiesReference element.*/
    public String getXadesNS(){
        return QPRElem.getNamespaceURI();
    }
    
    /**Retrievers the xades prefix from this QualifyingPropertiesReference element.*/
    public String getXadesPrefix(){
        return QPRElem.getPrefix();
    }  
    
    /**Set the Id. Not empty (may be null).*/
    public void setId(String id){
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may not be null (but may be empty.");
        }
        DOMUtils.setAttributeID(this.QPRElem, TAG_ID, id);
    }
    
    /**Returns the Id, or null if not specified.*/
    public String getId(){
        return DOMUtils.getAttributeValue(this.QPRElem, TAG_ID);
    }
    
    /**Returns the URI.*/
    public URI getURI()  throws MarshalException{
        return XSDAnyURI.parseURI(this.QPRElem.getAttributeNS(null, TAG_URI));
    }
    
    /**Returns this <QualifyingPropertiesReference>
     */
    public Element getElement(){
        return this.QPRElem;
    }
    
     /**Returns the qualifying properties references of a given XMLSignature 
     *Returns an empty list if not specified (never null.
     */
    public static List<DOMQualifyingPropertiesReference> 
            getQualifyingPropertiesReferences(XMLSignature sig,
            DOMCryptoContext context)
            throws MarshalException{
        List<DOMQualifyingPropertiesReference> refs = 
                new ArrayList<DOMQualifyingPropertiesReference>();
        List<DOMXMLObject> objs = (List<DOMXMLObject>) sig.getObjects();
        Iterator<DOMXMLObject> it = objs.iterator();
        while(it.hasNext()){
            DOMXMLObject obj = it.next();
            List<javax.xml.crypto.dom.DOMStructure> xmlstructs =
                    (List<javax.xml.crypto.dom.DOMStructure>)obj.getContent();
            Iterator<javax.xml.crypto.dom.DOMStructure> itStruct = xmlstructs.iterator();
            while(itStruct.hasNext()){
                Node node = itStruct.next().getNode();
                if(node.getNodeType()==Node.ELEMENT_NODE){
                    Element elem = (Element)node;
                    if(elem.getLocalName().equals(TAG_QUALIFYINGPROPERTIESREFERENCE)){
                        refs.add(new DOMQualifyingPropertiesReference(elem,
                                context));
                    }
                }
                
            }
            //because the references must be in the same <Object> element, 
            //we can return if one on more refs are found.
            if(refs.size()>0){
                return refs;
            }
        }
        return null;
    }
    public String toString(){
       try{
           String dump = "";
           String id = this.getId();
           if(id!=null){
               dump += "attribute Id="+id+"\n";
           }
           dump += "URI:"+this.getURI()+"\n";
           return dump;
       } catch(Exception e){
           e.printStackTrace();
           return "ERROR";
       }
        
    }
}
