/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDAnyURI;
import be.cardon.xml.XSDBase64Binary;
import java.net.URI;
import java.security.InvalidAlgorithmParameterException;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.spec.DigestMethodParameterSpec;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;

import javax.xml.crypto.dsig.DigestMethod;
import be.cardon.dsig.dom.DOMDigestMethod;


/**DOM-based implementation of {@code ReferenceInfoType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMReferenceInfo extends DOMStructure{
    
    public final static String TAG_DIGESTMETHOD = "DigestMethod";
    public final static String TAG_DIGESTVALUE = "DigestValue";
    public final static String TAG_ID = "Id";
    public final static String TAG_URI = "URI";
    
    /**The digest method. not null*/
    DOMDigestMethod digestMethod;
    
    /**The digest value, not null.*/
    byte[] digestValue;
    
    /**The optional URI, may be null.*/
    URI uri;
    
    /**The optional Id, may be null, but not empty.*/
    String id;
    
    /** Creates a new instance of DOMReferenceInfo.
     *@param digestMethod the digest method, not null.
     *@param digestValue the digest value, not null
     *@param URI the URI, optional (may be null)
     *@param Id the Id, optional (may be null, but not empty)
     *@throws NullPointerException if digestMethod or digestValue is null.
     *@throws XMLSignatureException if Id is empty.
     */
    public DOMReferenceInfo(
            DigestMethod digestMethod,
            byte[] digestValue,
            URI uri,
            String id) {
        if(digestMethod==null){
            throw new NullPointerException("digestMethod may not be null");
        }
        if(digestValue==null){
            throw new NullPointerException("digestValue may not be null");
        }
        if(id!=null && id.length()==0){
            throw new IllegalArgumentException("id may not be empty (but may be null)");
        }
        this.digestMethod = (DOMDigestMethod)digestMethod;
        this.digestValue = digestValue;
        this.uri = uri;
        this.id = id;
    }
    
    /**Unmarshal a {@code ReferenceInfo} element.*/
    public DOMReferenceInfo(Element el, DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(el, null, xadesNS);
        
        Element digestMethodElem = DOMUtils.getFirstChildElement(el);
        DOMUtils.checkName(digestMethodElem, TAG_DIGESTMETHOD, xadesNS);
        this.digestMethod = (DOMDigestMethod)DOMDigestMethod.unmarshal(digestMethodElem);
        
        Element digestValueElem = DOMUtils.getNextSiblingElement(digestMethodElem);
        DOMUtils.checkName(digestValueElem, TAG_DIGESTVALUE, xadesNS);
        this.digestValue = XSDBase64Binary.decode(
                digestValueElem.getTextContent());
        
        this.id = DOMUtils.getAttributeValue(el,TAG_ID); //returns null if absent
        this.uri = XSDAnyURI.parseURI(el.getAttributeNS(null, TAG_URI)); //null if empty uri;
        
    }
    public void marshal(
            Node parent, String localName,
            DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element referenceInfoElem = DOMUtils.createElement(doc,
                localName, xadesNS, xadesPrefix);
        
        digestMethod.marshal(referenceInfoElem, dsPrefix, context);
        
        Element digestValueElem = DOMUtils.createElement(
                doc, TAG_DIGESTVALUE, xadesNS, dsPrefix);
        digestValueElem.appendChild(doc.createTextNode(
                XSDBase64Binary.encode(digestValue)));
        referenceInfoElem.appendChild(digestValueElem);
        
        DOMUtils.setAttributeID(referenceInfoElem, TAG_ID, id); //only if id!=null
        
        if(uri!=null){
            referenceInfoElem.setAttributeNS(null, TAG_URI, 
                    XSDAnyURI.printURI(uri));
        }
        parent.appendChild(referenceInfoElem);
    }
    
    /**Returns the digest method. Never null.*/
    public DOMDigestMethod getDigestMethod() {
        return digestMethod;
    }
    
    /**Returns the digest value (never null).*/
    public byte[] getDigestValue() {
        return digestValue;
    }
    /**Return id, or null if absent.*/
    public String getId() {
        return id;
    }

    public URI getUri() {
        return uri;
    }

    public String toString(){
            try{   
            String dump="";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            if(this.getUri()!=null){
                dump += "attribute URI="+this.getUri()+"\n";
            }
            dump += "DigestMethod: "+this.getDigestMethod()+"\n";
            dump += "DigestValue (base64): "+XSDBase64Binary.encode(this.getDigestValue());
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
    
}
