/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;

/**DOM-based implementation of {@code RevocationValues}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMRevocationValues {
    
    public final static String TAG_CRLVALUES = "CRLValues";
    public final static String TAG_OCSPVALUES = "OCSPValues";
    public final static String TAG_OTHERVALUES = "OtherValues";
    public final static String TAG_ID = "Id";
    public final static String TAG_ENCAPSULATEDCRLVALUE = "EncapsulatedCRLValue";
    public final static String TAG_ENCAPSULATEDOCSPVALUE = "EncapsulatedOCSPValue";
    public final static String TAG_OTHERVALUE = "OtherValue";
    
    /**CRLValues, list may be empty or null.*/
    List<DOMEncapsulatedPKIData> cRLValues=new ArrayList<DOMEncapsulatedPKIData>();
    
    /**OCSPValues, list may be empty or null.*/
    List<DOMEncapsulatedPKIData> oCSPValues=new ArrayList<DOMEncapsulatedPKIData>();
    
    /**OtherValues, list may be empty or null.*/
    List<DOMAnyType> otherValues=new ArrayList<DOMAnyType>();
    
    /**Id, optional (may be null but not empty)*/
    String id;
    
    /** Creates a new instance of DOMRevocationValues 
     *@param CRLvalues
     *@param OCSPValues
     *@param otherValues
     *@param id
     *@throws IllegalArgumentException if id is an empty string 
     * (it may be null, but not empty).
     */
    public DOMRevocationValues(
            List<DOMEncapsulatedPKIData> cRLValues,
            List<DOMEncapsulatedPKIData> oCSPValues,
            List<DOMAnyType> otherValues,
            String id) {
        if(cRLValues!=null){
            this.cRLValues = cRLValues;
        }
        if(oCSPValues!=null){
            this.oCSPValues = oCSPValues;
        }
        if(id!=null){
            if(id.length()<1){
                throw new IllegalArgumentException("id may not be empty, but may be null");
            }
        }
        this.id = id;
    }
    
    public DOMRevocationValues(Element elem, DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        Element child = DOMUtils.getFirstChildElement(elem);
        
        if(child!=null && child.getLocalName().equals(TAG_CRLVALUES)){
            DOMUtils.checkName(child, null, xadesNS);
            Element encapsulatedCRLValueElem = DOMUtils.
                    getFirstChildElement(child);
            if(encapsulatedCRLValueElem==null){
                throw new MarshalException("CRLValues has no any element.");
            }
            while(encapsulatedCRLValueElem!=null){
                DOMUtils.checkName(encapsulatedCRLValueElem, 
                        TAG_ENCAPSULATEDCRLVALUE, xadesNS);
                this.cRLValues.add(new DOMEncapsulatedPKIData(
                        encapsulatedCRLValueElem,context));
                encapsulatedCRLValueElem = DOMUtils.getNextSiblingElement(
                        encapsulatedCRLValueElem);
            }
            child = DOMUtils.getNextSiblingElement(child);
        }
        
        if(child!=null && child.getLocalName().equals(TAG_CRLVALUES)){
            DOMUtils.checkName(child, null, xadesNS);
            Element encapsulatedOCSPValueElem = DOMUtils.
                    getFirstChildElement(child);
            if(encapsulatedOCSPValueElem==null){
                throw new MarshalException("OCSPValues has no any element.");
            }
            while(encapsulatedOCSPValueElem!=null){
                this.oCSPValues.add(new DOMEncapsulatedPKIData(
                        encapsulatedOCSPValueElem, context));
                encapsulatedOCSPValueElem = DOMUtils.getNextSiblingElement(
                        encapsulatedOCSPValueElem);
            }
            child = DOMUtils.getNextSiblingElement(child);
        }
                
        if(child!=null && child.getLocalName().equals(TAG_OTHERVALUES)){
            DOMUtils.checkName(child, null, xadesNS);
            Element otherValueElem = DOMUtils.
                    getFirstChildElement(child);
            if(otherValueElem==null){
                throw new MarshalException("OtherValues has no any element.");
            }
            while(otherValueElem!=null){
                this.otherValues.add(new DOMAnyType(
                        otherValueElem, context));
                otherValueElem = DOMUtils.getNextSiblingElement(
                        otherValueElem);
            }
            child = DOMUtils.getNextSiblingElement(child);
        }       
        if(child!=null){
            throw new MarshalException("Invalid RevovationValues element.");
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID);
    }
    
    public void marshal(Node parent, String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element revocationValuesElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        
        if(cRLValues.size()>0){
            Element cRLValuesElem = DOMUtils.createElement(
                doc, TAG_CRLVALUES, xadesNS, xadesPrefix);
            Iterator<DOMEncapsulatedPKIData> it = cRLValues.iterator();
            while(it.hasNext()){
                it.next().marshal(cRLValuesElem,
                        TAG_ENCAPSULATEDCRLVALUE, context);
            }
            revocationValuesElem.appendChild(cRLValuesElem);
        }
  
         if(oCSPValues.size()>0){
            Element oCSPValuesElem = DOMUtils.createElement(
                doc, TAG_OCSPVALUES, xadesNS, xadesPrefix);
            Iterator<DOMEncapsulatedPKIData> it = oCSPValues.iterator();
            while(it.hasNext()){
                it.next().marshal(oCSPValuesElem,
                        TAG_ENCAPSULATEDOCSPVALUE, context);
            }
            revocationValuesElem.appendChild(oCSPValuesElem);
        }
        
        if(otherValues.size()>0){
            Element otherValuesElem = DOMUtils.createElement(
                doc, TAG_OTHERVALUES, xadesNS, xadesPrefix);
            Iterator<DOMAnyType> it = otherValues.iterator();
            while(it.hasNext()){
                it.next().marshal(otherValuesElem,
                        TAG_OTHERVALUE, context);
            }
            revocationValuesElem.appendChild(otherValuesElem);
        }
        DOMUtils.setAttributeID(revocationValuesElem, TAG_ID, id); //do nothing if null
        parent.appendChild(revocationValuesElem);
    }

    public List<DOMEncapsulatedPKIData> getCRLValues() {
        return cRLValues;
    }

    public String getId() {
        return id;
    }

    public List<DOMEncapsulatedPKIData> getOCSPValues() {
        return oCSPValues;
    }

    public List<DOMAnyType> getOtherValues() {
        return otherValues;
    }

}
