/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.net.URI;

/**DOM-based implementation of {@code DOMSPUserNotice}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSPUserNotice extends DOMSigPolicyQualifier{
    
    public final static String TAG_SPUSERNOTICE = "SPUserNotice";
    public final static String TAG_NOTICEREF = "NoticeRef";
    public final static String TAG_EXPLICITTEXT = "ExplicitText";
    public final static String TAG_ORGANIZATION = "Organization";
    public final static String TAG_NOTICENUMBERS = "NoticeNumbers";
    public final static String TAG_INT = "int";
    
    /**NoticeRef, may be null.*/
    NoticeReference noticeRef;
    
    /**explicitText, never null but may be empty.*/
    String explicitText;
    
    
    public DOMSPUserNotice(NoticeReference noticeRef, String explicitText){
        super((NodeList)null); //bypass DOMAnyType
        this.noticeRef = noticeRef;
        this.explicitText = (explicitText==null) ? new String() : explicitText;
    }
    
    public DOMSPUserNotice(Element elem, DOMCryptoContext context)
    throws MarshalException{
        super(elem.getChildNodes());
        try{
            String xadesNS = XadESContext.getXadesNS(context);
            DOMUtils.checkName(elem, null, xadesNS);
            if(elem.getChildNodes().getLength()!=1){
                throw new MarshalException("policy qualifier contains more than one element.");
            }
            Element sPUserNoticeElem = DOMUtils.getFirstChildElement(elem);
            DOMUtils.checkName(sPUserNoticeElem, TAG_SPUSERNOTICE, xadesNS);
            
            Element child = DOMUtils.getFirstChildElement(sPUserNoticeElem);
            if(child!=null && child.getLocalName().equals(TAG_NOTICEREF)){
                DOMUtils.checkName(child, null, xadesNS);
                
                Element organizationElem = DOMUtils.getFirstChildElement(child);
                DOMUtils.checkName(organizationElem, TAG_ORGANIZATION, xadesNS);
                String organization = organizationElem.getTextContent();
                
                Element noticeNumbersElem = DOMUtils.getNextSiblingElement(organizationElem);
                DOMUtils.checkName(noticeNumbersElem, TAG_NOTICENUMBERS, xadesNS);
                List<Integer> noticeNumbers = new ArrayList<Integer>();
                Element intElem = DOMUtils.getFirstChildElement(noticeNumbersElem);
                while(intElem!=null){
                    DOMUtils.checkName(intElem, TAG_INT, xadesNS);
                    String intString = intElem.getTextContent();
                    Integer integer = new Integer(intString);
                    noticeNumbers.add(integer);
                    intElem = DOMUtils.getNextSiblingElement(intElem);
                }
                
                this.noticeRef = new NoticeReference(organization, noticeNumbers);
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child!=null && child.getLocalName().equals(TAG_EXPLICITTEXT)){
                DOMUtils.checkName(child, null, xadesNS);
                this.explicitText = child.getTextContent();
                child = DOMUtils.getNextSiblingElement(child);
            }
            if(child!=null){
                 throw new MarshalException("invalid SPUserNotice");
            }
        }catch(NumberFormatException e){
            throw new MarshalException(e);
        }
    }
    
    public void marshal(Node parent,
            String localName, DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element anyElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        Element sPUserNoticeElem =  DOMUtils.createElement(
                doc, TAG_SPUSERNOTICE, xadesNS, xadesPrefix);
        if(noticeRef!=null){
            Element noticeRefElem = DOMUtils.createElement(
                doc, TAG_NOTICEREF, xadesNS, xadesPrefix);
            Element organizationElem = DOMUtils.createElement(
                doc, TAG_ORGANIZATION, xadesNS, xadesPrefix);
            organizationElem.appendChild(
                    doc.createTextNode(noticeRef.getOrganization()));
            noticeRefElem.appendChild(organizationElem);
            Element noticeNumbersElem = DOMUtils.createElement(
                doc, TAG_NOTICENUMBERS, xadesNS, xadesPrefix);
            Iterator<Integer> it = noticeRef.getNoticeNumbers().iterator();
            while(it.hasNext()){
                Element intElem = DOMUtils.createElement(
                doc, TAG_INT, xadesNS, xadesPrefix);
                intElem.appendChild(doc.createTextNode(it.next().toString()));
                noticeNumbersElem.appendChild(intElem);
            }
            noticeRefElem.appendChild(noticeNumbersElem);
            sPUserNoticeElem.appendChild(noticeRefElem);
        }
        if(explicitText.length()>0){
            Element explicitTextELem = DOMUtils.createElement(
                doc, TAG_EXPLICITTEXT, xadesNS, xadesPrefix);
            explicitTextELem.appendChild(doc.createTextNode(explicitText));
            sPUserNoticeElem.appendChild(explicitTextELem);
        }
        anyElem.appendChild(sPUserNoticeElem);
        parent.appendChild(anyElem);
    }

    public String getExplicitText() {
        return explicitText;
    }

    public DOMSPUserNotice.NoticeReference getNoticeRef() {
        return noticeRef;
    }


    
    public static class NoticeReference{
        
        /**Organization, never null.*/
        String organization=new String();
        
        /**NoticeNumber, may be empty, but not null.*/
        List<Integer> noticeNumbers = new ArrayList<Integer>();
        
        public NoticeReference(String organization, List<Integer> noticeNumbers){
            if(organization!=null){
                this.organization = organization;
            }
            if(noticeNumbers!=null){
                this.noticeNumbers = noticeNumbers;
            }
        }
        
        public List<Integer> getNoticeNumbers() {
            return noticeNumbers;
        }
        
        public String getOrganization() {
            return organization;
        }
        
    }
    public String toString(){
        try{   
            String dump="";
            NoticeReference ref = this.getNoticeRef();
            if(ref!=null){
                dump += " NoticeReference:\n";
                dump += "  Organization: "+ref.getOrganization()+"\n";
                dump += "  NoticeNumbers: {";
                List<Integer> numbs = ref.getNoticeNumbers();
                for(int i=0; i<numbs.size()-1;i++){
                    dump += numbs.get(i)+",";
                }
                if(numbs.size()>0){
                    dump += numbs.get(numbs.size()-1);
                }
                dump +="}\n";
            }
            dump += " ExplicitText: "+this.getExplicitText()+"\n";
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
