/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import be.cardon.dsig.dom.DOMTransform;
import java.security.Provider;
import javax.xml.crypto.XMLCryptoContext;

/**DOM-based implementation of {@code SignaturePolicyIdentifierType}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSignaturePolicyIdentifier extends DOMStructure{
    
    public final static String TAG_SIGNATUREPOLICYID = "SignaturePolicyId";
    public final static String TAG_SIGNATUREPOLICYIMPLIED = "SignaturePolicyImplied";
    public final static String TAG_SIGPOLICYID = "SigPolicyId";
    public final static String TAG_SIGPOLICYHASH = "SigPolicyHash";
    public final static String TAG_SIGPOLICYQUALIFIERS = "SigPolicyQualifiers";
    public final static String TAG_SIGPOLICYQUALIFIER = "SigPolicyQualifier";
    public final static String TAG_TRANSFORMS = "Transforms";
    
    
    /**The SignaturePolicyId. When null, signature policy is implied.*/
    SignaturePolicyId signaturePolicyId;
    
    
    /** Creates a new instance of DOMSignaturePolicyIdentifier, with implied policy.
     *
     */
    public DOMSignaturePolicyIdentifier() {
        this.signaturePolicyId = null;
    }
    /*Creates a new instance of DOMSignaturePolicyIdentifier that is not implied.
     */
    public DOMSignaturePolicyIdentifier(
            DOMObjectIdentifier sigPolicyId,
                List<DOMTransform> transforms,
                DOMDigestAlgAndValue sigPolicyHash,
                List<DOMSigPolicyQualifier> sigPolicyQualifiers) {
        this.signaturePolicyId = new SignaturePolicyId(sigPolicyId,transforms,
                sigPolicyHash,sigPolicyQualifiers);
    }
    
    /**Unmarshal a SignaturePolicyIdentifier.*/
    public DOMSignaturePolicyIdentifier(Element elem,
            DOMCryptoContext context)
            throws MarshalException{
        try{
             String xadesNS = XadESContext.getXadesNS(context);
             DOMUtils.checkName(elem, null, xadesNS);
            
            Element child = DOMUtils.getFirstChildElement(elem);
            if(child.getLocalName().equals(TAG_SIGNATUREPOLICYID)){
                DOMUtils.checkName(child, null, xadesNS);
                
                Element sigPolicyIdElem = DOMUtils.getFirstChildElement(child);
                DOMUtils.checkName(sigPolicyIdElem, TAG_SIGPOLICYID, xadesNS);
                DOMObjectIdentifier sigPolicyId = new DOMObjectIdentifier(
                        sigPolicyIdElem, context);
                
                Element nextElem = DOMUtils.getNextSiblingElement(sigPolicyIdElem);
                List<DOMTransform> transforms=new ArrayList<DOMTransform>();
                if(nextElem.getLocalName().equals(TAG_TRANSFORMS)){
                    DOMUtils.checkName(nextElem, null, XMLSignature.XMLNS);
                    
                    Element transformElem = DOMUtils.getFirstChildElement(nextElem);
                    while(transformElem!=null){
                        transforms.add(new DOMTransform(transformElem, context,
                                XadESContext.getXMLSignatureProvider(context)));
                        transformElem = DOMUtils.getNextSiblingElement(transformElem);
                    }
                    nextElem = DOMUtils.getNextSiblingElement(nextElem);
                }
                Element sigPolicyHashElem = nextElem;
                DOMUtils.checkName(sigPolicyHashElem, TAG_SIGPOLICYHASH, xadesNS);
                DOMDigestAlgAndValue sigPolicyHash = new DOMDigestAlgAndValue(
                        sigPolicyHashElem,context);
                List<DOMSigPolicyQualifier> sigPolicyQualifiers =
                        new ArrayList<DOMSigPolicyQualifier>();
                
                Element sigPolicyQualifiersElem = DOMUtils.
                        getNextSiblingElement(sigPolicyHashElem); //may be null
                if(sigPolicyQualifiersElem!=null){
                    DOMUtils.checkName(sigPolicyQualifiersElem, 
                            TAG_SIGPOLICYQUALIFIERS, xadesNS);
                    Element sigPolicyQualifierElem = DOMUtils.getFirstChildElement(
                            sigPolicyQualifiersElem);
                    while(sigPolicyQualifierElem!=null){
                        DOMUtils.checkName(sigPolicyQualifierElem, TAG_SIGPOLICYQUALIFIER,
                                xadesNS);
                        sigPolicyQualifiers.add(DOMSigPolicyQualifier.unmarshal(
                                sigPolicyQualifierElem, context));
                        sigPolicyQualifierElem = DOMUtils.getNextSiblingElement(
                                sigPolicyQualifierElem);
                    }
                    if(sigPolicyQualifiers.size()==0){
                    throw new IllegalArgumentException("sigPolicyQualifiers is empty.");
                }
                }
                signaturePolicyId = new SignaturePolicyId(
                        sigPolicyId,
                        transforms,
                        sigPolicyHash,
                        sigPolicyQualifiers);
            }else{
                //must be implied
                DOMUtils.checkName(child, TAG_SIGNATUREPOLICYIMPLIED, xadesNS);
                signaturePolicyId = null;
            }
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }
    
    public void marshal(
            Node parent, String localName,
            DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element signaturePolicyIdentifierElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        if(this.isPolicyImplied()){
            Element signaturePolicyImpliedElem = DOMUtils.createElement(
                doc, TAG_SIGNATUREPOLICYIMPLIED, xadesNS, xadesPrefix);
            signaturePolicyIdentifierElem.appendChild(signaturePolicyImpliedElem);
        }else{
            Element signaturePolicyIdElem = DOMUtils.createElement(
                doc, TAG_SIGNATUREPOLICYID, xadesNS, xadesPrefix);
            signaturePolicyId.getSigPolicyId().marshal(
                    signaturePolicyIdElem, 
                    TAG_SIGPOLICYID, context);
            if(signaturePolicyId.getTransforms().size()>0){
                Element transformsElem = DOMUtils.createElement(
                    doc, TAG_TRANSFORMS, xadesNS, xadesPrefix);
                Iterator<DOMTransform> it = signaturePolicyId.
                        getTransforms().iterator();
                while(it.hasNext()){
                    it.next().marshal(transformsElem, dsPrefix, context);
                }
                signaturePolicyIdElem.appendChild(transformsElem);
            }
            signaturePolicyId.getSigPolicyHash().marshal(signaturePolicyIdElem, 
                    TAG_SIGPOLICYHASH, context);
            if(signaturePolicyId.getSigPolicyQualifiers().size()>0){
                Element sigPolicyQualifiersElem = DOMUtils.createElement(
                    doc, TAG_SIGPOLICYQUALIFIERS, xadesNS, xadesPrefix);
                Iterator<DOMSigPolicyQualifier> it = signaturePolicyId.
                        getSigPolicyQualifiers().iterator();
                while(it.hasNext()){
                    it.next().marshal(sigPolicyQualifiersElem, 
                            TAG_SIGPOLICYQUALIFIER, context);
                }
                signaturePolicyIdElem.appendChild(sigPolicyQualifiersElem);
            }
            signaturePolicyIdentifierElem.appendChild(signaturePolicyIdElem);
        }
        parent.appendChild(signaturePolicyIdentifierElem);
    }
    
    
    /**Returns true if the policy is implied.*/
    public boolean isPolicyImplied(){
        return (signaturePolicyId==null) ? true : false;
    }

    /**returns the SignaturePolicyId. When null, signature policy is implied.*/
    public DOMSignaturePolicyIdentifier.SignaturePolicyId getSignaturePolicyId() {
        return signaturePolicyId;
    }

    
    public static class SignaturePolicyId{
        
        /**the policy object identifier. Never null.*/
        DOMObjectIdentifier sigPolicyId;
        
        /**transforms, may be empty, but not null*/
        List<DOMTransform> transforms = new ArrayList<DOMTransform>();
        
        /**SigPolicyHash, never null.*/
        DOMDigestAlgAndValue sigPolicyHash;
        
        /**SigPolicyQualifiers, never null, may be empty.*/
        List<DOMSigPolicyQualifier> sigPolicyQualifiers =
                new ArrayList<DOMSigPolicyQualifier>();
        
        public SignaturePolicyId(
                DOMObjectIdentifier sigPolicyId,
                List<DOMTransform> transforms,
                DOMDigestAlgAndValue sigPolicyHash,
                List<DOMSigPolicyQualifier> sigPolicyQualifiers){
            if(sigPolicyId==null){
                throw new NullPointerException("SigPolicyHash cannot be null");
            }
            this.sigPolicyId = sigPolicyId;
            if(transforms!=null){
                this.transforms = transforms;
            }
            if(sigPolicyHash==null){
                throw new NullPointerException("SigPolicyHash cannot be null");
            }
            this.sigPolicyHash = sigPolicyHash;
            if(sigPolicyQualifiers!=null){
                this.sigPolicyQualifiers = sigPolicyQualifiers;
            }
           
        }
        
        public DOMDigestAlgAndValue getSigPolicyHash() {
            return sigPolicyHash;
        }
        
        public List<DOMSigPolicyQualifier> getSigPolicyQualifiers() {
            return sigPolicyQualifiers;
        }
        
        public List<DOMTransform> getTransforms() {
            return transforms;
        }

        public DOMObjectIdentifier getSigPolicyId() {
            return sigPolicyId;
        }
       public String toString(){
          try{   
            String dump="";
            dump += "[SigPolicyId:\n"+ this.getSigPolicyId()+"]\n";
            Iterator it= this.getTransforms().iterator();
            while(it.hasNext()){
                dump += "[Transform:\n"+it.next()+"]\n";
            }
            dump += "[SigPolicyHash:\n"+this.getSigPolicyHash()+"]\n";
            it = this.getSigPolicyQualifiers().iterator();
            while(it.hasNext()){
                dump += "[SigPolicyQualifier:\n"+it.next()+"]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
    }
    public String toString(){
        try{   
            if(this.isPolicyImplied()){
                return "SignaturePolicyImplied";
            }
            return this.getSignaturePolicyId().toString();
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
