/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import be.cardon.dsig.dom.DOMTransform;
import javax.xml.crypto.XMLCryptoContext;
import java.security.Provider;

/**DOM-based implementation of {@code SignedDataObjectProperties}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSignedDataObjectProperties extends DOMStructure {
    public final static String TAG_DATAOBJECTFORMAT = "DataObjectFormat";
    public final static String TAG_COMMITMENTTYPEINDICATION = "CommitmentTypeIndication";
    public final static String TAG_ALLDATAOBJECTSTIMESTAMP = "AllDataObjectsTimeStamp";
    public final static String TAG_INDIVIDUALDATAOBJECTSTIMESTAMP = "IndividualDataObjectsTimeStamp";
    public final static String TAG_ID = "Id";
    
    /**never null, may be empty.*/
    List<DOMDataObjectFormat> dataObjectFormats =
            new ArrayList<DOMDataObjectFormat>();
    
    /**never null, may be empty.*/
    List<DOMCommitmentTypeIndication> commitmentTypeIndications =
            new ArrayList<DOMCommitmentTypeIndication>();
    
    /**never null, may be empty.*/
    List<DOMAllDataObjectsTimeStamp> allDataObjectsTimeStamps =
            new ArrayList<DOMAllDataObjectsTimeStamp>();
    
    /**never null, may be empty.*/
    List<DOMXadESTimeStamp> individualDataObejctsTimeStamps =
            new ArrayList<DOMXadESTimeStamp>();
    
    /**may be null but not empty.*/
    String id;
    
    /** Creates a new instance of DOMSignedDataObjectProperties */
    public DOMSignedDataObjectProperties(
            List<DOMDataObjectFormat> dataObjectFormats,
            List<DOMCommitmentTypeIndication> commitmentTypeIndications,
            List<DOMAllDataObjectsTimeStamp> allDataObjectsTimeStamps,
            List<DOMXadESTimeStamp> individualDataObejctsTimeStamps,
            String id) {
        if(dataObjectFormats!=null){
            this.dataObjectFormats = dataObjectFormats;
        }
        if(commitmentTypeIndications!=null){
            this.commitmentTypeIndications = commitmentTypeIndications;
        }
        if(allDataObjectsTimeStamps!=null){
            this.allDataObjectsTimeStamps = allDataObjectsTimeStamps;
        }
        if(individualDataObejctsTimeStamps!=null){
            this.individualDataObejctsTimeStamps = individualDataObejctsTimeStamps;
        }
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("id may not be null.");
        }
        this.id = id;
    }
    
    public DOMSignedDataObjectProperties(Element elem, 
            DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        DOMUtils.checkName(elem, null, xadesNS);
        Element child = DOMUtils.getFirstChildElement(elem);
        
        while(child!=null && child.getLocalName().equals(TAG_DATAOBJECTFORMAT)){
            this.dataObjectFormats.add(new DOMDataObjectFormat(child, context));
            child = DOMUtils.getNextSiblingElement(child);
        }
        
        while(child!=null && child.getLocalName().equals(TAG_COMMITMENTTYPEINDICATION)){
            this.commitmentTypeIndications.add(new 
                    DOMCommitmentTypeIndication(child, context));
            child = DOMUtils.getNextSiblingElement(child);
        }

        while(child!=null && child.getLocalName().equals(TAG_ALLDATAOBJECTSTIMESTAMP)){
            this.allDataObjectsTimeStamps.add(new 
                    DOMAllDataObjectsTimeStamp(child, context));
            child = DOMUtils.getNextSiblingElement(child);
        }
        
        while(child!=null && child.getLocalName().equals(TAG_INDIVIDUALDATAOBJECTSTIMESTAMP)){
            this.individualDataObejctsTimeStamps.add(new 
                    DOMXadESTimeStamp(child, context));
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null){
            throw new MarshalException("Invalid SignedDataObjectProperties");
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID); //may be null;
        
    }

    public List<DOMXadESTimeStamp> getIndividualDataObejctsTimeStamps() {
        return individualDataObejctsTimeStamps;
    }

    public String getId() {
        return id;
    }

    public List<DOMDataObjectFormat> getDataObjectFormats() {
        return dataObjectFormats;
    }

    public List<DOMCommitmentTypeIndication> getCommitmentTypeIndications() {
        return commitmentTypeIndications;
    }

    public List<DOMAllDataObjectsTimeStamp> getAllDataObjectsTimeStamps() {
        return allDataObjectsTimeStamps;
    }

    public void marshal(Node parent, 
                String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element signedDataObjectPropertiesElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
 
        if(dataObjectFormats.size()>0){
            Iterator<DOMDataObjectFormat> it = dataObjectFormats.iterator();
            while(it.hasNext()){
                it.next().marshal(signedDataObjectPropertiesElem,
                        TAG_DATAOBJECTFORMAT, context);
            }
        }

        if(commitmentTypeIndications.size()>0){
            Iterator<DOMCommitmentTypeIndication> it = commitmentTypeIndications.iterator();
            while(it.hasNext()){
                it.next().marshal(signedDataObjectPropertiesElem,
                        TAG_COMMITMENTTYPEINDICATION, context);
            }
        }

        if(allDataObjectsTimeStamps.size()>0){
            Iterator<DOMAllDataObjectsTimeStamp> it = allDataObjectsTimeStamps.iterator();
            while(it.hasNext()){
                it.next().marshal(signedDataObjectPropertiesElem,null);
            }
        }

        if(individualDataObejctsTimeStamps.size()>0){
            Iterator<DOMXadESTimeStamp> it = individualDataObejctsTimeStamps.iterator();
            while(it.hasNext()){
                it.next().marshal(signedDataObjectPropertiesElem,null);
            }
        }
        DOMUtils.setAttributeID(signedDataObjectPropertiesElem, TAG_ID, id);
        parent.appendChild(signedDataObjectPropertiesElem);
    }
    
    public String toString(){
        try{
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            Iterator it;
            it = this.getDataObjectFormats().iterator();
            while(it.hasNext()){
                dump += "[DataObjectFormat:\n" + it.next() + "]\n";
            }

            it = this.getCommitmentTypeIndications().iterator();
            while(it.hasNext()){
                dump += "[CommitmentTypeIndication:\n" + it.next() + "]\n";
            }
            
            it = this.getAllDataObjectsTimeStamps().iterator();
            while(it.hasNext()){
                dump += "[getAllDataObjectsTimeStamp:\n" + it.next() + "]\n";
            }
            
            it = this.getIndividualDataObejctsTimeStamps().iterator();
            while(it.hasNext()){
                dump += "[IndividualDataObejctsTimeStamps:\n" + it.next() + "]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
