/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMCanonicalizationMethod;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import java.security.Provider;
import javax.xml.crypto.XMLCryptoContext;

/**DOM-based implementation of {@code SignedProperties}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSignedProperties extends DOMStructure{
    
    public final static String TAG_SIGNEDSIGNATUREPROPERTIES = "SignedSignatureProperties";
    public final static String TAG_SIGNEDDATAOBJECTPROPERTIES = "SignedDataObjectProperties";
    public final static String TAG_ID = "Id";
    
    /**not null.*/
    DOMSignedSignatureProperties signedSignatureProperties;
    
    /**may be null.*/
    DOMSignedDataObjectProperties signedDataObjectProperties;
    
    /**may be null, but not empty.*/
    String id;
    
    /** Creates a new instance of DOMSignedProperties.
     *@param signedSignatureProperties, required.
     *@param signedDataObjectProperties, optional (may be null)
     *@param id, may be null but not empty
     *@throws NullPointerException if signedSignatureProperties is null.
     *@throws IllegalArgumentException if id is null.
     */
    public DOMSignedProperties(
            DOMSignedSignatureProperties signedSignatureProperties,
            DOMSignedDataObjectProperties signedDataObjectProperties,
            String id) {
        if(signedSignatureProperties==null){
            throw new NullPointerException("SignedSignatureProperties is null.");
        }
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may not be empty.");
        }
        
        this.signedSignatureProperties=signedSignatureProperties;
        this.signedDataObjectProperties=signedDataObjectProperties;
        this.id=id;
    }
    
    public DOMSignedProperties(Element elem, 
            DOMCryptoContext context)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        DOMUtils.checkName(elem, null, xadesNS);
        
        Element signedSignaturePropertiesElem = DOMUtils.getFirstChildElement(elem);
        DOMUtils.checkName(signedSignaturePropertiesElem, 
                TAG_SIGNEDSIGNATUREPROPERTIES, xadesNS);
        this.signedSignatureProperties = new DOMSignedSignatureProperties(
                signedSignaturePropertiesElem, context);
        
        Element signedDataObjectPropertiesElem = DOMUtils.getNextSiblingElement(
                signedSignaturePropertiesElem);
        if(signedDataObjectPropertiesElem!=null){
            DOMUtils.checkName(signedDataObjectPropertiesElem, 
                    TAG_SIGNEDDATAOBJECTPROPERTIES, xadesNS);
            this.signedDataObjectProperties = new DOMSignedDataObjectProperties(
                    signedDataObjectPropertiesElem, context);
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID);
    }
    
    /**marshal the element as first child.*/
    public void marshal(Node parent, 
                String localName, DOMCryptoContext context)
        throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element signedPropertiesElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        
        signedSignatureProperties.marshal(signedPropertiesElem, 
                TAG_SIGNEDSIGNATUREPROPERTIES, context);
        
        if(signedDataObjectProperties!=null){
            signedDataObjectProperties.marshal(signedPropertiesElem, 
                TAG_SIGNEDDATAOBJECTPROPERTIES, context);
        }
        DOMUtils.setAttributeID(signedPropertiesElem, TAG_ID, id);
        Node firstChildOfParent = parent.getFirstChild();
        parent.insertBefore(signedPropertiesElem, firstChildOfParent);
    }

    public String getId() {
        return id;
    }

    public DOMSignedDataObjectProperties getSignedDataObjectProperties() {
        return signedDataObjectProperties;
    }

    public DOMSignedSignatureProperties getSignedSignatureProperties() {
        return signedSignatureProperties;
    }
    
    public String toString(){
        try{
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            if(this.getSignedSignatureProperties()!=null){
                dump += "[SignedSignatureProperties:\n" + this.getSignedSignatureProperties().toString() 
                + "] end SignedSignatureProperties\n";
            }
            if(this.getSignedDataObjectProperties()!=null){
                dump += "[SignedDataObjectProperties:\n" + this.getSignedDataObjectProperties().toString() 
                + "] end SignedDataObjectProperties\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }

}
