/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.xml.XSDDateTime;
import java.util.GregorianCalendar;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.XMLCryptoContext;
import java.security.Provider;

/**DOM-based implementation of {@code SignedSignatureProperties}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSignedSignatureProperties extends DOMStructure{
    
    public final static String TAG_SIGNINGTIME = "SigningTime";
    public final static String TAG_SIGNINGCERTIFICATE = "SigningCertificate";
    public final static String TAG_SIGNATUREPOLICYIDENTIFIER = "SignaturePolicyIdentifier";
    public final static String TAG_SIGNATUREPRODUCTIONPLACE = "SignatureProductionPlace";
    public final static String TAG_SIGNERROLE = "SignerRole";
    public final static String TAG_ID = "Id";
    
    /**SigningTime. may be null.*/
    GregorianCalendar signingTime;
    
    /**SigningCertificate. may be null.*/
    DOMCertIDList signingCertificate;
    
    /**SignaturePolicyIdentifier, may be null.*/
    DOMSignaturePolicyIdentifier signaturePolicyIdentifier;
    
    /**SignatureProductionPlace, may be null.*/
    DOMSignatureProductionPlace signatureProductionPlace;
    
    /**SignerRole, may be null.*/
    DOMSignerRole signerRole;
    
    /**Id. Mey be null but not empty.*/
    String id;
    
    /** Creates a new instance of DOMSignedSignatureProperies.
     *@param signingTime, optional (may be null)
     *@param signingCertificate, optional (may be null)
     *@param signaturePolicyIdentifier, optional (may be null)
     *@param signatureProductionPlace, optional (may be null)
     *@param signatureRole, optional (may be null)
     *@param id, optional (may be null, but not empty)
     *@throws IllegalArgumentException if id is empty.
     */
    public DOMSignedSignatureProperties(
            GregorianCalendar signingTime,
            DOMCertIDList signingCertificate,
            DOMSignaturePolicyIdentifier signaturePolicyIdentifier,
            DOMSignatureProductionPlace signatureProductionPlace,
            DOMSignerRole signerRole,
            String id) {
        this.signingTime=signingTime;
        this.signingCertificate=signingCertificate;
        this.signaturePolicyIdentifier=signaturePolicyIdentifier;
        this.signatureProductionPlace=signatureProductionPlace;
        this.signerRole=signerRole;
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may not be empty (but can be null");
        }
        this.id=id;
    }
    
    /**Constructor without any data.*/
    public DOMSignedSignatureProperties(String id){
        setId(id);
    }
    
    public DOMSignedSignatureProperties(Element elem, 
            DOMCryptoContext context )
            throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        DOMUtils.checkName(elem, null, xadesNS);
        Element child = DOMUtils.getFirstChildElement(elem);
        if(child!=null && child.getLocalName().equals(TAG_SIGNINGTIME)){
            DOMUtils.checkName(child, null, xadesNS);
            this.signingTime = XSDDateTime.parseDateTime(child.getTextContent()).
                    toGregorianCalendar();
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_SIGNINGCERTIFICATE)){
            DOMUtils.checkName(child, null, xadesNS);
            this.signingCertificate = new DOMCertIDList(child, context);
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_SIGNATUREPOLICYIDENTIFIER)){
            DOMUtils.checkName(child, null, xadesNS);
            this.signaturePolicyIdentifier = new DOMSignaturePolicyIdentifier(
                    child, context);
            child = DOMUtils.getNextSiblingElement(child);
        }
        
        if(child!=null && child.getLocalName().equals(TAG_SIGNATUREPRODUCTIONPLACE)){
            DOMUtils.checkName(child, null, xadesNS);
            this.signatureProductionPlace = new DOMSignatureProductionPlace(child, context);
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_SIGNERROLE)){
            DOMUtils.checkName(child, null, xadesNS);
            this.signerRole = new DOMSignerRole(child, context);
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null){
            throw new MarshalException("Invalid SignedSignatureProperties.");
        }
        this.id = DOMUtils.getAttributeValue(elem, TAG_ID);
    }
    
    public void marshal(Node parent, 
            String localName, DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element signedSignaturePropertiesElem = DOMUtils.createElement(
                doc, localName, xadesNS, xadesPrefix);
        
        if(signingTime!=null){
            Element signingTimeElem = DOMUtils.createElement(
                    doc, TAG_SIGNINGTIME, xadesNS, xadesPrefix);
            signingTimeElem.appendChild(doc.createTextNode(
                    XSDDateTime.printDateTime(signingTime)));
            signedSignaturePropertiesElem.appendChild(signingTimeElem);
        }
        if(signingCertificate!=null){
            signingCertificate.marshal(signedSignaturePropertiesElem,
                    TAG_SIGNINGCERTIFICATE, context);
        }
        if(signaturePolicyIdentifier!=null){
            signaturePolicyIdentifier.marshal(signedSignaturePropertiesElem,
                   TAG_SIGNATUREPOLICYIDENTIFIER, context);
        }
        if(signatureProductionPlace!=null){
            signatureProductionPlace.marshal(signedSignaturePropertiesElem,
                    TAG_SIGNATUREPRODUCTIONPLACE, context);
        }
        if(signerRole!=null){
            signerRole.marshal(signedSignaturePropertiesElem,
                    TAG_SIGNERROLE, context);
        }
        DOMUtils.setAttributeID(signedSignaturePropertiesElem, TAG_ID, id); //only if not null;
        parent.appendChild(signedSignaturePropertiesElem);
    }
    
    public String getId() {
        return id;
    }
    
    public DOMSignaturePolicyIdentifier getSignaturePolicyIdentifier() {
        return signaturePolicyIdentifier;
    }
    
    public DOMSignatureProductionPlace getSignatureProductionPlace() {
        return signatureProductionPlace;
    }
    
    public DOMSignerRole getSignerRole() {
        return signerRole;
    }
    
    public DOMCertIDList getSigningCertificate() {
        return signingCertificate;
    }
    
    public GregorianCalendar getSigningTime() {
        return signingTime;
    }
    
    public void setSignaturePolicyIdentifier(DOMSignaturePolicyIdentifier signaturePolicyIdentifier) {
        this.signaturePolicyIdentifier = signaturePolicyIdentifier;
    }
    
    public void setId(String id) {
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("id may not be empty (but can be null.");
        }
        this.id = id;
    }
    
    public void setSignatureProductionPlace(DOMSignatureProductionPlace signatureProductionPlace) {
        this.signatureProductionPlace = signatureProductionPlace;
    }
    
    public void setSignerRole(DOMSignerRole signerRole) {
        this.signerRole = signerRole;
    }
    
    public void setSigningCertificate(DOMCertIDList signingCertificate) {
        this.signingCertificate = signingCertificate;
    }
    
    public void setSigningTime(GregorianCalendar signingTime) {
        this.signingTime = signingTime;
    }
    public String toString(){
        try{
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            if(this.getSigningTime()!=null){
                dump += "[SigningTime: " + this.getSigningTime().toString() + "]\n";
            }
            if(this.getSigningCertificate()!=null){
                dump += "[SigningCertificate:\n" + this.getSigningCertificate().toString() + "]\n";
            }
            if(this.getSignaturePolicyIdentifier()!=null){
                dump += "[SignaturePolicyIdentifier:\n" + this.getSignaturePolicyIdentifier().toString() + "]\n";
            }
            if(this.getSignatureProductionPlace()!=null){
                dump += "[SignatureProductionPlace:\n" + this.getSignatureProductionPlace().toString() + "]\n";
            }
            if(this.getSignerRole()!=null){
                dump += "[SignerRole:\n" + this.getSignerRole().toString() + "]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}