/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;

/**DOM-based implementation of {@code SignerRole}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMSignerRole extends DOMStructure{
    
    public final static String TAG_CLAIMEDROLES = "ClaimedRoles";
    public final static String TAG_CERTIFIEDROLES = "CertifiedRoles";
    public final static String TAG_CLAIMEDROLE = "ClaimedRole";
    public final static String TAG_CERTIFIEDROLE = "CertifiedRole";
    
    /**claimedRoles, never null, may be empty.*/
    List<DOMClaimedRole> claimedRoles = 
            new ArrayList<DOMClaimedRole>();
    
    /**certifiedRoles, never null, may be empty.*/
     List<DOMEncapsulatedPKIData> certifiedRoles = 
             new ArrayList<DOMEncapsulatedPKIData>();
    
    /** Creates a new instance of SignerRole.*
     *the entries may be null.
     */
    public DOMSignerRole(
            List<DOMClaimedRole> claimedRoles, 
            List<DOMEncapsulatedPKIData> certifiedRoles) {
        if(claimedRoles!=null){
            this.claimedRoles = claimedRoles;
        }
        if(certifiedRoles!=null){
            this.certifiedRoles = certifiedRoles;                    
        }
    }
    
    /**Unmarshall a SignerRole.*/
    public DOMSignerRole(Element elem, DOMCryptoContext context)
    throws MarshalException{
        try{
            String xadesNS = XadESContext.getXadesNS(context);
            DOMUtils.checkName(elem, null, xadesNS);
            Element child = DOMUtils.getFirstChildElement(elem);
            if(child != null && child.getLocalName().equals(TAG_CLAIMEDROLES)){
                DOMUtils.checkName(child, null, xadesNS);
                Element claimedRoleElem = DOMUtils.getFirstChildElement(child);
                while(claimedRoleElem!=null){
                    DOMUtils.checkName(claimedRoleElem, TAG_CLAIMEDROLE, xadesNS);
                    DOMClaimedRole claimedRole = 
                            DOMClaimedRole.unmarshall(claimedRoleElem, context);
                    claimedRoles.add(claimedRole);
                    claimedRoleElem = DOMUtils.getNextSiblingElement(claimedRoleElem);
                }
                if(claimedRoles.size()<1){
                    throw new MarshalException("ClaimedRoles size < 1");
                }
                child = DOMUtils.getNextSiblingElement(child);
            }
            
            if(child!=null && child.getLocalName().equals(TAG_CERTIFIEDROLES)){
                DOMUtils.checkName(child, null, xadesNS);
                Element certifiedRoleElem = DOMUtils.getFirstChildElement(child);
                while(certifiedRoleElem!=null){
                    DOMUtils.checkName(certifiedRoleElem, TAG_CERTIFIEDROLE, xadesNS);
                    DOMEncapsulatedPKIData certifiedRole = 
                            new DOMEncapsulatedPKIData(certifiedRoleElem, context);
                    certifiedRoles.add(certifiedRole);
                    certifiedRoleElem = DOMUtils.getNextSiblingElement(certifiedRoleElem);
                }
                if(certifiedRoles.size()<1){
                    throw new MarshalException("CertifiedRoles size < 1");
                }
                child = DOMUtils.getNextSiblingElement(child);
            }
            
            if(child !=null){
                throw new MarshalException("Invalid SignerRole element.");
            }
            
        }catch(NullPointerException e){
            throw new MarshalException(e);
        }
    }

    public void marshal(Node parent,
            String localName,  DOMCryptoContext context)
            throws MarshalException{
        Document doc = DOMUtils.getOwnerDocument(parent);
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Element signerRoleElem = DOMUtils.createElement(doc,
                localName, xadesNS, xadesPrefix);
        if(claimedRoles.size()>0){
            Element claimedRolesElem = DOMUtils.createElement(doc,
                TAG_CLAIMEDROLES, xadesNS, xadesPrefix);
            Iterator<DOMClaimedRole> it = claimedRoles.iterator();
            while(it.hasNext()){
                DOMClaimedRole claimedRole = it.next();
                claimedRole.marshal(claimedRolesElem,
                        TAG_CLAIMEDROLE, context);
            }
            signerRoleElem.appendChild(claimedRolesElem);
        }
        
        
        if(certifiedRoles.size()>0){
            Element certifiedRolesElem = DOMUtils.createElement(doc,
                TAG_CERTIFIEDROLES, xadesNS, xadesPrefix);
            Iterator<DOMEncapsulatedPKIData> it = certifiedRoles.iterator();
            while(it.hasNext()){
                DOMEncapsulatedPKIData certifiedRole = it.next();
                certifiedRole.marshal(certifiedRolesElem, 
                        TAG_CERTIFIEDROLE, context);
            }
            signerRoleElem.appendChild(certifiedRolesElem);
        }
        
        parent.appendChild(signerRoleElem);
    }

    public List<DOMClaimedRole> getClaimedRoles() {
        return claimedRoles;
    }

    public List<DOMEncapsulatedPKIData> getCertifiedRoles() {
        return certifiedRoles;
    }

    public String toString(){
        try{   
            String dump = "";
            Iterator it = this.getClaimedRoles().iterator();
            while(it.hasNext()){
                dump += "[ClaimedRole:\n"+it.next()+"]\n";
            }
            it = this.getCertifiedRoles().iterator();
            while(it.hasNext()){
                dump += "[CertifiedRoles:\n"+it.next()+"]\n";
            }
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
}
