/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMXMLSignature;
import com.sun.org.apache.xpath.internal.functions.Function;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.XMLCryptoContext;
import java.security.Provider;
import org.w3c.dom.NodeList;

/**DOM-based implementation of {@code UnsignedProperties}.
 *
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMUnsignedProperties {
    
    public final static String TAG_UNSIGNEDPROPERTIES = "UnsignedProperties";
    public final static String TAG_UNSIGNEDSIGNATUREPROPERTIES = "UnsignedSignatureProperties";
    public final static String TAG_UNSIGNEDDATAOBJECTPROPERTIES = "UnsignedDataObjectProperties";
    public final static String TAG_ID = "Id";
    
    /**The <UnsignedProperties> element. Never null.
     */
    Element UPElem;
    
   
    /**The crypto context.*/
    DOMCryptoContext context;

    
    /** Creates a new instance of DOMUnsignedProperties */
    public DOMUnsignedProperties(String id, Document doc,
            DOMCryptoContext context)
            throws IllegalArgumentException{
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        if(!xadesNS.equals(Constants.Xades132SpecNS)){
            throw new IllegalArgumentException("Unvalid XadES version: "+xadesNS);
        }
        this.UPElem = DOMUtils.createElement(
                doc, TAG_UNSIGNEDPROPERTIES,
                xadesNS, xadesPrefix);
        
       
        this.context = context;

        
        this.setId(id);
    }
    
    /**Unmarshal a UnsignedProperties.
     *@param elem the element to unmarshall
     *@param xadesNS the XadES namespace
     *@param context the cryptoContext
     *@param provider the provider
     *@param dsPrefix the prefix of the XMLSignature namepsace
     *@param xadesPrefix the prefix of the XadES namespace
     */
    public DOMUnsignedProperties(Element elem, 
            DOMCryptoContext context)
            throws MarshalException{
        this.UPElem = elem;
        this.context = context;
        String xadesNS = XadESContext.getXadesNS(context);
       DOMUtils.checkName(elem, TAG_UNSIGNEDPROPERTIES, xadesNS);
        
        Element child = DOMUtils.getFirstChildElement(elem);
        if(child!=null && child.getLocalName().equals(TAG_UNSIGNEDSIGNATUREPROPERTIES)){
            DOMUtils.checkName(child, null, xadesNS);
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null && child.getLocalName().equals(TAG_UNSIGNEDDATAOBJECTPROPERTIES)){
            DOMUtils.checkName(child, null, xadesNS);
            child = DOMUtils.getNextSiblingElement(child);
        }
        if(child!=null){
            throw new MarshalException("Invalid SignedDataObject");
        }
    }
    
    /**Set the Id. Not empty (may be null).*/
    public void setId(String id){
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may not be null (but may be empty.");
        }
        DOMUtils.setAttributeID(this.UPElem, TAG_ID, id);
    }
    
    /**Returns the Id, or null if not specified.*/
    public String getId(){
        return DOMUtils.getAttributeValue(this.UPElem, TAG_ID);
    }
    
    /**Returns the unsigned signature properties or null if not specified.*/
    public DOMUnsignedSignatureProperties getUnsignedSignatureProperties()
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        Element unsignedSignaturePropertiesElem = (Element) UPElem.getElementsByTagNameNS(
                xadesNS, TAG_UNSIGNEDSIGNATUREPROPERTIES).item(0);
        //return null if time > length
        if(unsignedSignaturePropertiesElem!=null){
            return new DOMUnsignedSignatureProperties(unsignedSignaturePropertiesElem,
                    context);
        }else{
            return null;
        }
    }
    
    
    /**Returns the unsigned data object properties or null if not specified.*/
    public DOMUnsignedDataObjectProperties getUnsignedDataObjectProperties()
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        Element unsignedDataObjectPropertiesElem = (Element) UPElem.getElementsByTagNameNS(
                xadesNS, TAG_UNSIGNEDDATAOBJECTPROPERTIES).item(0);
        //return null if time > length
        if(unsignedDataObjectPropertiesElem!=null){
            return new DOMUnsignedDataObjectProperties(
                    unsignedDataObjectPropertiesElem,
                    context);
        }else{
            return null;
        }
    }
    
    /**Set the UnsignedDataObjectProperties with the given one. If there is
     *already such element, the given element overwrite the old element.*/
    public void setUnsignedDataObjectProperties(
            DOMUnsignedDataObjectProperties unsignedDataObjectProperties)
            throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        Element unsignedDataObjectPropertiesElem = (Element) UPElem.getElementsByTagNameNS(
                xadesNS, TAG_UNSIGNEDDATAOBJECTPROPERTIES).item(0);
        //return null if time > length
        if(unsignedDataObjectPropertiesElem!=null){
            //replace it:
            UPElem.removeChild(unsignedDataObjectPropertiesElem);
        }
        unsignedDataObjectProperties.marshal(UPElem,
                TAG_UNSIGNEDDATAOBJECTPROPERTIES, context); //always last
    }
    
    /**return the unsignedSignatureProperties or create a new one if it doesn't exists.
     *Returns never null.
     */
    public DOMUnsignedSignatureProperties getOrCreateUnsignedSignatureProperties()
    throws MarshalException{
        DOMUnsignedSignatureProperties usp = getUnsignedSignatureProperties();
        if(usp==null){
            String xadesNS = XadESContext.getXadesNS(context);
            usp = new DOMUnsignedSignatureProperties(null,
                    UPElem.getOwnerDocument(), 
                    context);
            Element unsignedDataObjectPropertiesElem = (Element) UPElem.
                    getElementsByTagNameNS(
                    xadesNS, TAG_UNSIGNEDDATAOBJECTPROPERTIES).item(0); //null is not exists.
            this.UPElem.insertBefore(usp.USPElem, unsignedDataObjectPropertiesElem);
        }
        return usp;
    }
    
    public String toString(){
        try{   
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            if(this.getUnsignedSignatureProperties()!=null){
                dump += "[UnsignedSignatureProperties:\n" + this.getUnsignedSignatureProperties().toString() 
                + "] end UnsignedSignatureProperties \n";
            }
            if(this.getUnsignedDataObjectProperties()!=null){
                dump += "[UnsignedDataObjectProperties:\n" + this.getUnsignedDataObjectProperties().toString() 
                + "] end UnsignedDataObjectProperties\n";
            }
            return dump;
        } catch(MarshalException e){
            e.printStackTrace();
            return "Marshal ERROR";
        }
    }
    
}
