/*
Copyright (c) 2007, CARDON DE LICHTBUER Rodolphe
All rights reserved.
 
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 
1. Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.
 
2. Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 
3. The name of the author or contributors may not be used to endorse or promote
products derived from this software without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
 
 */

package be.cardon.xades;

import be.cardon.dsig.dom.DOMXMLSignature;
import be.cardon.dsig.dom.DOMCanonicalizationMethod;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dom.DOMCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.XMLCryptoContext;
import java.security.Provider;

/**DOM-based implementation of {@code UnsignedSignatureProperties}.
 *This class is a 'proxy' class. You can directly add (or remove)
 * properties on an existing UnsignedSignatureProperties element.
 * @author CARDON DE LICHTBUER Rodolphe
 */
public class DOMUnsignedSignatureProperties{
    
    public final static String TAG_DS_SIGNATURE = "Signature";
    public final static String TAG_UNSIGNEDSIGNATUREPROPERTIES = "UnsignedSignatureProperties";
    public final static String TAG_COUNTERSIGNATURE = "CounterSignature";
    public final static String TAG_SIGNATURETIMESTAMP = "SignatureTimeStamp";
    public final static String TAG_COMPLETECERTIFICATEREFS = "CompleteCertificateRefs";
    public final static String TAG_COMPLETEREVOCATIONREFS = "CompleteRevocationRefs";
    public final static String TAG_ATTRIBUTECERTIFICATEREFS = "AttributeCertificateRefs";
    public final static String TAG_ATTRIBUTEREVOCATIONREFS = "AttributeRevocationRefs";
    public final static String TAG_SIGANDREFSTIMESTAMP = "SigAndRefsTimeStamp";
    public final static String TAG_REFSONLYTIMESTAMP = "RefsOnlyTimeStamp";
    public final static String TAG_CERTIFICATEVALUES = "CertifivateValues";
    public final static String TAG_REVOCATIONVALUES = "RevocationValues";
    public final static String TAG_ATTRAUTHORITIESCERTVALUES = "AttrAuthoritiesCertValues";
    public final static String TAG_ATTRIBUTEREVOCATIONVALUES = "AttributeRevocationValues";
    public final static String TAG_ARCHIVETIMESTAMP = "ArchiveTimeStamp";
    public final static String TAG_ID = "Id";
    
    /**List with the knwown tags of children od UnsignedSignatureProperties element.*/
    public static List<String> KnownChildrenTags;
    
    static{
        KnownChildrenTags = new ArrayList<String>();
        KnownChildrenTags.add(TAG_COUNTERSIGNATURE);
        KnownChildrenTags.add(TAG_SIGNATURETIMESTAMP);
        KnownChildrenTags.add(TAG_COMPLETECERTIFICATEREFS);
        KnownChildrenTags.add(TAG_COMPLETEREVOCATIONREFS);
        KnownChildrenTags.add(TAG_ATTRIBUTECERTIFICATEREFS);
        KnownChildrenTags.add(TAG_ATTRIBUTEREVOCATIONREFS);
        KnownChildrenTags.add(TAG_SIGANDREFSTIMESTAMP);
        KnownChildrenTags.add(TAG_REFSONLYTIMESTAMP);
        KnownChildrenTags.add(TAG_CERTIFICATEVALUES);
        KnownChildrenTags.add(TAG_REVOCATIONVALUES);
        KnownChildrenTags.add(TAG_ATTRAUTHORITIESCERTVALUES);
        KnownChildrenTags.add(TAG_ATTRIBUTEREVOCATIONVALUES);
        KnownChildrenTags.add(TAG_ARCHIVETIMESTAMP);
    }
    
    
    
    /**The UnsignedSignatureProperties element. Never null.
     */
    Element USPElem;
    
    /**The crypto context.*/
    DOMCryptoContext context;
        
    /** Creates a new instance of DOMUnsignedSignatureProperties
     *@param id the optional Id
     *@param doc the DOM document
     *@param context the CryptoContext
     *@throws IllegalArgumentException if incorrect version.
     */
    public DOMUnsignedSignatureProperties(String id, Document doc,
             DOMCryptoContext context)
            throws IllegalArgumentException{
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        if(!xadesNS.equals(Constants.Xades132SpecNS)){
            throw new IllegalArgumentException("Unvalid XadES version: "+xadesNS);
        }
        this.USPElem = DOMUtils.createElement(
                doc, TAG_UNSIGNEDSIGNATUREPROPERTIES,
                xadesNS, xadesPrefix);
        

        this.context = context;
      
        this.setId(id);
        
    }
    
    /**Unmarshal a UnsignedSignatureProperties.
     *@param elem the element to unmarshall
     *@param xadesNS the XadES namespace
     *@param context the cryptoContext
     *@param provider the provider
     *@param dsPrefix the prefix of the XMLSignature namepsace
     *@param xadesPrefix the prefix of the XadES namespace
     */
    public DOMUnsignedSignatureProperties(Element elem,
            DOMCryptoContext context)
    throws MarshalException{
        this.USPElem = elem;
        this.context = context;
        String xadesNS = XadESContext.getXadesNS(context);
     
        //structure check => TO DO
        DOMUtils.checkName(elem, null, xadesNS);
    }
    
    /**Set the Id. Not empty (may be null).*/
    public void setId(String id){
        if(id!=null && id.length()<1){
            throw new IllegalArgumentException("Id may not be null (but may be empty.");
        }
        DOMUtils.setAttributeID(this.USPElem, TAG_ID, id);
    }
    
    /**Returns the Id, or null if not specified.*/
    public String getId(){
        return DOMUtils.getAttributeValue(this.USPElem, TAG_ID);
    }
    
    /**Returns the counter signatures.*/
    public Iterator<XMLSignature> getCounterSignatures(){
        return  (Iterator<XMLSignature>) new UnsignedPropertiesIterator(
                TAG_COUNTERSIGNATURE,  context);        
    }
    
    /**Returns the signature time stamps.*/
    public Iterator<DOMXadESTimeStamp> getSignatureTimeStamps(){
        return (Iterator<DOMXadESTimeStamp>)  new UnsignedPropertiesIterator(
                TAG_SIGNATURETIMESTAMP, context);   
    }
   
        /**Returns the list of (complete certificate references).*/
    public Iterator<DOMCompleteCertificateRefs> getCompleteCertificateRefss(){
        return (Iterator<DOMCompleteCertificateRefs>)  new UnsignedPropertiesIterator(
                TAG_COMPLETECERTIFICATEREFS, context);   
    }
    
        /**Returns the list of (complete revocation references).*/
    public Iterator<DOMCompleteRevocationRefs> getCompleteRevocationRefss(){
        return (Iterator<DOMCompleteRevocationRefs>)  new UnsignedPropertiesIterator(
                TAG_COMPLETEREVOCATIONREFS, context);   
    }
    
        /**Returns the list of (attribute certificate references).*/
    public Iterator<DOMCompleteCertificateRefs> getAttributeCertificateRefss(){
        return (Iterator<DOMCompleteCertificateRefs>)  new UnsignedPropertiesIterator(
                TAG_ATTRIBUTECERTIFICATEREFS, context);   
    }
    
        /**Returns the list of (attribute revocation references).*/
    public Iterator<DOMCompleteRevocationRefs> getAtributeRevocationRefss(){
        return (Iterator<DOMCompleteRevocationRefs>)  new UnsignedPropertiesIterator(
                TAG_ATTRIBUTEREVOCATIONREFS, context);   
    }
    
        /**Returns the signature and references time stamps.*/
    public Iterator<DOMXadESTimeStamp> getSigAndRefsTimeStamps(){
        return (Iterator<DOMXadESTimeStamp>)  new UnsignedPropertiesIterator(
                TAG_SIGANDREFSTIMESTAMP, context);   
    }
    
        /**Returns the references only time stamps.*/
    public Iterator<DOMXadESTimeStamp> getRefsOnlyTimeStamps(){
        return (Iterator<DOMXadESTimeStamp>)  new UnsignedPropertiesIterator(
                TAG_REFSONLYTIMESTAMP, context);   
    }
        /**Returns the list of (certificates values).*/
    public Iterator<DOMCertificateValues> getCertificateValuess(){
        return (Iterator<DOMCertificateValues>)  new UnsignedPropertiesIterator(
                TAG_CERTIFICATEVALUES, context);   
    }
        /**Returns the list of (revocation values).*/
    public Iterator<DOMRevocationValues> getRevocationValuess(){
        return (Iterator<DOMRevocationValues>)  new UnsignedPropertiesIterator(
                TAG_REVOCATIONVALUES, context);   
    }
        /**Returns the list of (attribute authorities certValues).*/
    public Iterator<DOMCertificateValues> getAttrAuthoritiesCertValuess(){
        return (Iterator<DOMCertificateValues>)  new UnsignedPropertiesIterator(
                TAG_ATTRAUTHORITIESCERTVALUES, context);   
    }
        /**Returns the list of (attribute revocation values).*/
    public Iterator<DOMRevocationValues> getAttributeRevocationValuess(){
        return (Iterator<DOMRevocationValues>)  new UnsignedPropertiesIterator(
                TAG_ATTRIBUTEREVOCATIONVALUES,  context );   
    }
        /**Returns the archive time stamps.*/
    public Iterator<DOMXadESTimeStamp> getArchiveTimeStamps(){
        return (Iterator<DOMXadESTimeStamp>)  new UnsignedPropertiesIterator(
                TAG_ARCHIVETIMESTAMP, context);   
    }
        /**Returns all the other unsigned signature properties.*/
    public Iterator<DOMXadESTimeStamp> getOthers(){
        throw new IllegalArgumentException("not implemented yet");
    }
   
    /**Add a counter signature.*/
    public void addCounterSignature(DOMXMLSignature sig)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        Document doc = DOMUtils.getOwnerDocument(USPElem);
        Element counterSignatureElem = DOMUtils.createElement(doc,
                TAG_COUNTERSIGNATURE, xadesNS, xadesPrefix);
        sig.marshal(counterSignatureElem, dsPrefix, context);
        USPElem.appendChild(counterSignatureElem);
    }
    

    /**Add a signature time stamp.*/
    public void addSignatureTimeStamp(List<DOMInclude> includes,
            DOMCanonicalizationMethod canonicalizationMethod,
            List<DOMEncapsulatedPKIData> encapsulatedTimeStamps,
            List<DOMAnyType> xmlTimeStamps, String id)
    throws MarshalException{
        String xadesNS = XadESContext.getXadesNS(context);
        String xadesPrefix = XadESContext.getXadesNSPrefix(context);
        String dsPrefix = XadESContext.getDSIGNSPrefix(context);
        DOMXadESTimeStamp timeStamp = new DOMXadESTimeStamp(TAG_SIGNATURETIMESTAMP,
                includes, canonicalizationMethod, encapsulatedTimeStamps, xmlTimeStamps, id,
                this.USPElem.getOwnerDocument(), context);
        timeStamp.marshal(USPElem, null);
    }
    /**Add a complete certificate references list.*/
    public void addCompleteCertificateRefs(DOMCompleteCertificateRefs refs)
    throws MarshalException{
        refs.marshal(USPElem, 
                TAG_COMPLETECERTIFICATEREFS, context);
    }

    /**Add a complete revocation references list.*/
    public void addCompleteRevocationRefs(DOMCompleteRevocationRefs refs)
    throws MarshalException{
        refs.marshal(USPElem, 
                TAG_COMPLETEREVOCATIONREFS, context);
    }

    /**Add an attribute certificate references list.*/
    public void addAttributeCertificateRefs(DOMCompleteCertificateRefs refs)
    throws MarshalException{
        refs.marshal(USPElem, 
                TAG_ATTRIBUTECERTIFICATEREFS, context);
    }

    /**Add attribute revocation references.*/
    public void addAttributeRevocationRefs(DOMCompleteRevocationRefs refs)
    throws MarshalException{
        refs.marshal(USPElem, 
                TAG_ATTRIBUTEREVOCATIONREFS, context);
    }

    /**Add a signature and references time stamp.*/
    public void addSigAndRefsTimeStamp(List<DOMInclude> includes,
            DOMCanonicalizationMethod canonicalizationMethod,
            List<DOMEncapsulatedPKIData> encapsulatedTimeStamps,
            List<DOMAnyType> xmlTimeStamps,  String id)
    throws MarshalException{
        DOMXadESTimeStamp timeStamp = new DOMXadESTimeStamp(TAG_SIGANDREFSTIMESTAMP,
                includes, canonicalizationMethod, encapsulatedTimeStamps, xmlTimeStamps, id,
                this.USPElem.getOwnerDocument(),
                context);
        timeStamp.marshal(USPElem, null);
    }
    

    /**Add a references only time stamp.*/
    public void addRefsOnlyTimeStamp(List<DOMInclude> includes,
            DOMCanonicalizationMethod canonicalizationMethod,
            List<DOMEncapsulatedPKIData> encapsulatedTimeStamps,
            List<DOMAnyType> xmlTimeStamps,  String id)
    throws MarshalException{
        DOMXadESTimeStamp timeStamp = new DOMXadESTimeStamp(TAG_REFSONLYTIMESTAMP,
                includes, canonicalizationMethod, encapsulatedTimeStamps, xmlTimeStamps, id,
                this.USPElem.getOwnerDocument(),
                context);
        timeStamp.marshal(USPElem, null);
    }

    /**Add certificate values.*/
    public void addCertificateValues(DOMCertificateValues certValues)
    throws MarshalException{
        certValues.marshal(USPElem, 
                TAG_CERTIFICATEVALUES, context);
    }

    /**Add revocation values.*/
    public void addRevocationValues(DOMRevocationValues revocValues)
    throws MarshalException{
        revocValues.marshal(USPElem, 
                TAG_REVOCATIONVALUES, context);
    }

    /**Add attribute authorities certificate values.*/
    public void addAttrAuthoritiesCertValues(DOMCertificateValues certValues)
    throws MarshalException{
        certValues.marshal(USPElem, 
                TAG_ATTRAUTHORITIESCERTVALUES, context);
    }

    /**Add attribute revocation values.*/
    public void addAttributeRevocationValues(DOMRevocationValues revocValues)
    throws MarshalException{
        revocValues.marshal(USPElem, 
                TAG_ATTRIBUTEREVOCATIONVALUES, context);
    }

    /**Add an archive time stamp.*/
    public void addArchiveTimeStamp(List<DOMInclude> includes,
            DOMCanonicalizationMethod canonicalizationMethod,
            List<DOMEncapsulatedPKIData> encapsulatedTimeStamps,
            List<DOMAnyType> xmlTimeStamps,  String id)
    throws MarshalException{
        DOMXadESTimeStamp timeStamp = new DOMXadESTimeStamp(TAG_ARCHIVETIMESTAMP,
                includes, canonicalizationMethod, encapsulatedTimeStamps, xmlTimeStamps, id,
                this.USPElem.getOwnerDocument(), context);
        timeStamp.marshal(USPElem, null);
    }

    public void addOther(Element other)
    throws MarshalException{
        USPElem.appendChild(other);
    }


    public class UnsignedPropertiesIterator implements Iterator{
        
        String tag;
        DOMCryptoContext context;
        Element currentChild;
        Element nextChild;

        /**Creates a new iterator, that iterates over children of a given element.
	*@param tag if null, iterates on the 'others' properties (unknown)
	*/
        public UnsignedPropertiesIterator(String tag, DOMCryptoContext context){
            String xadesNS = XadESContext.getXadesNS(context);
            Element child = DOMUtils.getFirstChildElement(USPElem);
            while(child!=null &&
                    (    !child.getLocalName().equals(tag)
                    || !child.getNamespaceURI().equals(xadesNS) )){
                child = DOMUtils.getNextSiblingElement(child);
            }
            Element nextChild = child;
            while(nextChild!=null &&
                    (    !nextChild.getLocalName().equals(tag)
                    || !nextChild.getNamespaceURI().equals(xadesNS) )){
                nextChild = DOMUtils.getNextSiblingElement(nextChild);
            }
            this.currentChild = child;
            this.nextChild = nextChild;
            this.tag = tag;
            this.context = context;
        }
        
        public boolean hasNext(){
            return (this.nextChild!=null);
        }
        
        /**Returns null on error.*/
        public Object next()
        throws java.util.NoSuchElementException{
            try{
            String xadesNS = XadESContext.getXadesNS(context);
            Provider provider = XadESContext.getXMLSignatureProvider(context);
            if(!hasNext()){
                throw new java.util.NoSuchElementException();
            }
            this.currentChild = this.nextChild;
            Element child = DOMUtils.getNextSiblingElement(this.nextChild);
            while(child!=null &&
                    (    !child.getLocalName().equals(tag)
                    || !child.getNamespaceURI().equals(xadesNS) )){
                child = DOMUtils.getNextSiblingElement(child);
            }
            this.nextChild = child;
            if(tag.equals(TAG_COUNTERSIGNATURE)){
                Element dsSignatureElement = DOMUtils.getFirstChildElement(
                        this.currentChild);
                DOMUtils.checkName(dsSignatureElement, TAG_DS_SIGNATURE,
                        XMLSignature.XMLNS);
                return new DOMXMLSignature(dsSignatureElement, 
                        this.context, provider);
            }else if(tag.equals(TAG_SIGNATURETIMESTAMP)){
                return new DOMXadESTimeStamp(this.currentChild,
                        this.context );
            }else if(tag.equals(TAG_COMPLETECERTIFICATEREFS)){
                return new DOMCompleteCertificateRefs(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_COMPLETEREVOCATIONREFS)){
                return new DOMCompleteRevocationRefs(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_ATTRIBUTECERTIFICATEREFS)){
                return new DOMCompleteCertificateRefs(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_ATTRIBUTEREVOCATIONREFS)){
                return new DOMCompleteRevocationRefs(this.currentChild,
                        this.context);          
            }else if(tag.equals(TAG_SIGANDREFSTIMESTAMP)){
                return new DOMXadESTimeStamp(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_REFSONLYTIMESTAMP)){
                return new DOMCertificateValues(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_REVOCATIONVALUES)){
                return new DOMRevocationValues(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_ATTRAUTHORITIESCERTVALUES)){
                return new DOMRevocationValues(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_ATTRAUTHORITIESCERTVALUES)){
                return new DOMCertificateValues(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_ATTRIBUTEREVOCATIONVALUES)){
                return new DOMRevocationValues(this.currentChild,
                        this.context);
            }else if(tag.equals(TAG_ARCHIVETIMESTAMP)){
                return new DOMXadESTimeStamp(this.currentChild,
                        this.context);
            }else{ //other
                return new DOMAnyType(this.currentChild, this.context);
            }
            }catch(MarshalException e){
                e.printStackTrace();
                return null;
            }
        }
        public void remove(){
            USPElem.removeChild(this.currentChild);
        }
        
    }
    
    public String toString(){
        try{   
            String dump = "";
            if(this.getId()!=null){
                dump += "attribute Id="+this.getId()+"\n";
            }
            Iterator it;
            it = this.getCounterSignatures();
            while(it.hasNext()){
                dump += "[CounterSignature:\n" + it.next().toString() + "]\n";
            }
            it = this.getSignatureTimeStamps();
            while(it.hasNext()){
                dump += "[SignatureTimeStamp:\n" + it.next().toString() + "]\n";
            }
            it = this.getCompleteCertificateRefss();
            while(it.hasNext()){
                dump += "[CompeteCertificateRefs:\n" + it.next().toString() + "]\n";
            }
            it = this.getCompleteRevocationRefss();
            while(it.hasNext()){
                dump += "[CompleteRevocationRefs:\n" + it.next().toString() + "]\n";
            }
            it = this.getAttributeCertificateRefss();
            while(it.hasNext()){
                dump += "[AttributeCertificateRefs:\n" + it.next().toString() + "]\n";
            }
            it = this.getAtributeRevocationRefss();
            while(it.hasNext()){
                dump += "[AtributeRevocationRefs:\n" + it.next().toString() + "]\n";
            }
            it = this.getSigAndRefsTimeStamps();
            while(it.hasNext()){
                dump += "[SigAndRefsTimeStamp:\n" + it.next().toString() + "]\n";
            }
             it = this.getRefsOnlyTimeStamps();
            while(it.hasNext()){
                dump += "[RefsOnlyTimeStamp:\n" + it.next().toString() + "]\n";
            }           
            it = this.getCertificateValuess();
            while(it.hasNext()){
                dump += "[CertificateValues:\n" + it.next().toString() + "]\n";
            }            
            it = this.getRevocationValuess();
            while(it.hasNext()){
                dump += "[RevocationValues:\n" + it.next().toString() + "]\n";
            }   
            it = this.getAttrAuthoritiesCertValuess();
            while(it.hasNext()){
                dump += "[AttrAuthoritiesCertValues:\n" + it.next().toString() + "]\n";
            }   
            it = this.getAttributeRevocationValuess();
            while(it.hasNext()){
                dump += "[AttributeRevocationValues:\n" + it.next().toString() + "]\n";
            }  
            it = this.getArchiveTimeStamps();
            while(it.hasNext()){
                dump += "[ArchiveTimeStamp:\n" + it.next().toString() + "]\n";
            }              
            return dump;
        } catch(Exception e){
            e.printStackTrace();
            return "ERROR";
        }
    }
    
}
